<?php
/* AES.php
 *
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * | AlberT-EasySite: Web Site Management System                          |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; either version 2 of the License.       |
 * +----------------------------------------------------------------------+
 * | Authors: Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: AES.php,v 1.18.2.12 2003/02/17 21:46:46 AlberT Exp $
 */

define('AES_VERSION',    '1.0.0-a5');
define('AES_BUILT_DATE', '02-17-2003');

$AES_objects          = array();
$AES_objInstanceCount = array();
$AES_themes           = array();
$AES_parserActive     = TRUE;

// Include global configuration file containing default settings and
//  configurations not to be overwritten by others
$_thisDir = dirname(__FILE__);
require_once $_thisDir.'/globalConfig.inc.php';
require_once $_thisDir.'/lib/gettext_replacer.inc.php';
// if we are in Admin Panel return (we don't need anything else ...)
if (defined('AES_ADMIN') && AES_ADMIN===TRUE)
	return;

// Set language to user's one
if (is_array($_SERVER) && array_key_exists('HTTP_ACCEPT_LANGUAGE', $_SERVER)) {
	$lang = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
}
if (isset($_COOKIE['AES_lang']) || !empty($_COOKIE['AES_lang'])) {
	$lang = $_COOKIE['AES_lang'];
}
if (isset($_GET['lang'])) {
	$lang = $_GET['lang'];
}
if (!isset($lang))
	$lang = AES_DEFAULT_LANGUAGE;
putenv('LANG='.$lang);
$lang_err=FALSE;
if (FALSE === setlocale(LC_ALL, $lang))
	$lang_err=TRUE;
// Specify location of translation tables
bindtextdomain('messages', AES_INST_PATH.'/AES/locale');
textdomain('messages');
setcookie('AES_lang', $lang, time()+AES_COOKIE_VALIDITY, AES_COOKIE_PATH,
          AES_COOKIE_DOMAIN, 0);

require_once $_thisDir.'/modules/cache/AlberT-cache.inc.php';

include_once $_thisDir.'/handleErrors.inc.php';
if (defined('AES_DEBUG') && AES_DEBUG === TRUE) {
	AES_print('<br><div align="center">AES '.AES_VERSION.': <b>'.
	          _("DEBUG MODE ENABLED").'</b><br><font size="-2">('.
	          _("edit config.inc.php to disable").')</font></div>');
}
if ($lang_err===TRUE)
	AES_print('Failed setting language to '.$lang."\n");
require_once $_thisDir.'/findThemes.inc.php';

// Set theme to user's one
if (isset($_COOKIE['AES_theme']) || !empty($_COOKIE['AES_theme'])) {
	$theme = $_COOKIE['AES_theme'];
}
if (isset($_GET['theme'])) {
	$theme = $_GET['theme'];
}
if (!isset($theme) || !in_array($theme, $AES_themes)) {
	$theme = AES_DEFAULT_THEME;
}
setcookie('AES_theme', $theme, time()+AES_COOKIE_VALIDITY, AES_COOKIE_PATH,
          AES_COOKIE_DOMAIN, 0);

include_once $_thisDir.'/AESinfo.inc.php';
require_once $_thisDir.'/manageCallback.inc.php';
require_once $_thisDir.'/registerPlugins.inc.php';
require_once $_thisDir.'/loadPlugins.inc.php';
require_once $_thisDir.'/parseTemplate.inc.php';
require_once $_thisDir.'/managePackages.inc.php';
unset($_thisDir);

if (isset($obStart) && $obStart === TRUE)
	ob_start();

/*******************************************************************\
**
**      Class AES:
**
**    manage html and images
**
\*******************************************************************/

class AES extends _parseTemplate
{
	// "Private" variables
	var $_defaultPage = '';
	var $_template    = '';
	var $_theme       = '';
	var $obj          = NULL;

	// Constructor
	function AES($tmpl = '')
	{
		global $AES_conf, $AES_objects, $AES_objInstanceCount, $theme;

		$this->setTheme($theme);
		$this->_defaultPage = AES_SITE_PATH.'/themes/'.$this->_theme.
		                      '/defaultPage.thm.php';
		$this->_template = AES_SITE_PATH.'/themes/'.$this->_theme.'/'.
		                   $this->_theme.'.thm.php';
		if (!empty($tmpl)) {
			$this->_template = $tmpl;
		}

		// Make 'objects' accessible by AES
		$this->obj = &$AES_objects;
	}

	// Set theme and default page
	function setTheme($theme='')
	{
		if (!empty($theme))
			$this->_theme = $theme;
		$this->_defaultPage = (@is_readable(AES_SITE_PATH.'/themes/'.
		                                    $this->_theme.
		                                    '/defaultPage.thm.php') )
		                      ? AES_SITE_PATH.'/themes/'.$this->_theme.
		                        '/defaultPage.thm.php'
		                      : AES_print(sprintf(_("Default page for theme ".
		                                            "'%s' does not exist !"),
		                                  $this->_theme), E_ERROR);

		return TRUE;
	}

	// Set objects properties from user space
	function setObj($name, $inst, $parm)
	{
		global $AES_objects;

		$func = 'set_'.$name;
		$AES_objects[$name][$inst]->$func($parm);

		return TRUE;
	}

	// Show the HTML page
	function showHTML()
	{
		$this->getParsed($this->_template);

		return TRUE;
	}
}

?>
