<?php
/* functions.inc.php
 *
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * | AlberT-EasySite: Web Site Management System                          |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; either version 2 of the License.       |
 * +----------------------------------------------------------------------+
 * | Authors: Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: functions.inc.php,v 1.10.2.2 2003/02/07 21:54:51 AlberT Exp $
 */

function chkDate($data, $sep='/')
{
	$d = explode($sep, $data);
	if (!checkdate($d[1], $d[0], $d[2])) {
		 return FALSE;
	}
	return TRUE;
}

function chkDateStr($date, $sep='/')
{
	$d = explode($sep, $date);
	if (!checkdate($d[1], $d[0], $d[2])) {
		 return '<font color="#ff0000">*</font>';
	}
	return '';
}

/** mkSelect()
 *  generate an HTML select box
 *
 *  $name is the name of the select ($_POST['name']) will be avaible 
 *    after submition
 *  $options is an associative array of DISPLAYED_TXT => NAME_VALUE
 *  $defVal is an optional defualt selected value string
 *  $eccept is an optional string to exclude a value from to be selected
 *  $method defaults to POST, may be GET
 */
function mkSelect($name, &$options, $defVal='', $eccept='', $method='POST', 
                  $others='')
{
	// Printing results in HTML;
	$method = '_'.strtoupper($method);     // workaround for Superglobals
	$method =& $GLOBALS[$method];          // dynamic array construction
	echo '<select name="'.$name.'" '.$others.">\n";
	if (!is_array($defVal)) {
		$defVal =  array($defVal);
	}

	if (!is_array($options))
		$options = array();
	
	foreach ($options as $option => $val) {
		$selected = '';
		if (isset($method[$name]) 
		    && (!empty($method[$name]) || $method[$name]==='0')) {
			if ($val==stripslashes($method[$name])) {
				$selected = 'selected=""';
			}
		}
		elseif ((!empty($defVal) || $defVal!=='') 
		        && in_array($val, $defVal, TRUE)) {
			$selected = 'selected=""';
		}
		if ($val!==$eccept)
			echo '  <option '.$selected.' value="'.$val.'">'.$option."\n";
	}
	echo "</select>\n";
	return;
}

/** mkRadio()
 *  generate a set of HTML radios
 *
 *  $name is the name of the radio ($_POST['name']) will be avaible 
 *    after submition
 *  $options is an associative array of DISPLAYED_TXT => NAME_VALUE
 *  $defVal is an optional defualt selected value string
 *  $txtDx tells if text has to be on the right of radio or not
 *  $method defaults to POST, may be GET
 */
function mkRadio($name, &$options, $defVal='', $txtDx=TRUE, $method='POST')
{
	// Printing results in HTML
	$method = '_'.strtoupper($method);     // workaround for Superglobals
	$method =& $GLOBALS[$method];          // dynamic array construction
	if (!is_array($defVal)) {
		$defVal =  array($defVal);
	}
	foreach ($options as $option => $value) {
		$checked = '';
		if (isset($method[$name]) 
		    && (!empty($method[$name]) || $method[$name]==='0')) {
			if (stripslashes($method[$name])==$value)
				$checked = 'checked=""';
		}
		elseif (!empty($defVal) && in_array($value, $defVal, TRUE)) {
			$checked = 'checked=""';
		}
		if ($txtDx===FALSE)
			print($option.' <input type="radio" '.$checked.' name="'.$name.
			      '" value="'.$value.'" alt="'.$name.'">'."\n");
		else
			print('<input type="radio" '.$checked.' name="'.$name.'" value="'.
			      $value.'" alt="'.$name.'">'.$option."\n");
	}
}

/** teosophic()
 *  calculates the teosophic reduction of a number
 *
 *  calculate a recursive sum of each digit of the number:
 *  1234 -> 10 -> 1
 *
 *  @arg $num the number to be reduced
 */
function teosophic($num)
{
	$num = (string) $num;
	$len  = strlen($num);
	$sum = 0;

	if ($len<=1)
		return (int) $num;
	for ($i=0; $i<$len; $i++) {
		$sum += (int) $num[$i];
	}
	return teosophic($sum);
}

/** upload_files()
 *
 *  moves uploaded files to destination dir
 *
 *  @arg    string $dest_path the destination path
 *  @arg    string $prefix    an optional prefix used in destination file 
 *                            name generation
 *  @return mixed             array on success, FALSE on failure
 */
function upload_files($dest_path, $prefix='pre')
{
	if (!is_array($_FILES) || empty($_FILES))
		return FALSE;
	foreach ($_FILES as $file) {
		$files_names = array();

		$res = pathinfo($file['name']);
		$ext = isset($res['extension']) ? '.'.$res['extension'] : '';
		$dest_name = uniqid($prefix).$ext;
		$dest = $dest_path.'/'.$dest_name;
		if (!move_uploaded_file($file['tmp_name'], $dest))
			return FALSE;
		$files_names[] = $dest;
	}
	return $files_names;
}
?>
