<?php
/* loadPlugins.inc.php
 *
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * | AlberT-EasySite: Web Site Management System                          |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; either version 2 of the License.       |
 * +----------------------------------------------------------------------+
 * | Authors: Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: loadPlugins.inc.php,v 1.7.2.2 2003/01/13 01:37:25 AlberT Exp $
*/

///////////////////////////////////////////////////////////////////////////////
//
// function _AES_getPluginsDirName($plgDir)
//
///////////////////////////////////////////////////////////////////////////////
//
// Get the names of all dirs under "plugins" dir
//
///////////////////////////////////////////////////////////////////////////////
//
// Parameter:
// $plgDir - Plugins directory
//
///////////////////////////////////////////////////////////////////////////////
//
// Return value:
// 0 - error
// array - ok
//
///////////////////////////////////////////////////////////////////////////////
//
// Global references:
// $AES_ignoredPlugins
// $AES_disabledPlugins
//
///////////////////////////////////////////////////////////////////////////////

function &_AES_getPluginsDirName($directory)
{
	global $AES_ignoredPlugins, $AES_disabledPlugins;

	// directories like "\foo\08012001" (on win systems) bring to an error
	//  such a name, containing the special '\0' char, will terminate
	//  prematurely the string.
	$directory = str_replace ('\0', "\\0", $directory);
	$dh = @opendir ($directory);
	// Under Windows NT opendir() seems to return a resource even for
	//  a non-existent directory; we use is_dir to avoid it.
	if (!$dh) {
		if (!@is_dir($directory)) {
			AES_print( sprintf(_("Cannot open directory '%s'. ".
			           "It is NOT a directory!"), $directory) );
		}
		elseif (!@is_readable($directory)) {
			AES_print( sprintf(_("Cannot open directory '%s'. ".
			           "It MUST be readable (+r)!"), $directory) );
		}
		// check that dir il executable, if it isn't is_dir() used below
		//  doesn't works properly!
		elseif (!@is_executable($directory)) {
			AES_print( sprintf(_("Directory '%s' MUST be executable (+x)!"),
			                   $directory) );
		}
		AES_print( sprintf(_("Cannot open directory '%s'.. Unexpected error!"),
		                   $directory) );
	return FALSE;
	}
	$plugins = array();
	//this handle strange entries like a dir named '0'
	while (FALSE !== ($entry = @readdir ($dh))) {
		if ($entry != '.' && $entry != '..') {
			$dir = "$directory/$entry";
			// files will be ignored! Each plugin MUST have its own dir
			if (@is_dir($dir)) {
				if (@file_exists("$dir/AES_registerPlugin.inc.php") ) {
					if (!@in_array($entry, $AES_disabledPlugins, TRUE)) {
						$plugins[] = $entry;
					}
					else
						AES_print( sprintf(_("Plugin '<b>%s</b>' is present ".
						           "but <font color=\"red\">disabled</font>."),
						           $entry) );
				}
				else {
					$AES_ignoredPlugins[] = $entry;
					AES_print( sprintf(_("Directory <b>'%s'</b> seems NOT to ".
					           "be a valid AES plugin dir: it DOES NOT ".
					           "contain the file AES_registerPlugin.inc.php ".
					           "or this file is NOT readable from the web ".
					           "server !!!"), $dir) );
				}
			}
		}
	}
	@closedir ($dh);
	return $plugins ;
}
//}} _AES_getPluginsDirName()

if (defined('AES_ADMIN') && AES_ADMIN===TRUE)
	return;


/*
 * Load plugins in includer directory, if it's different from AES
 */
if (AES_SITE_PATH !== AES_INST_PATH) {
	$pluginDirs =& _AES_getPluginsDirName(AES_SITE_PATH.'/plugins');
	if (!empty($pluginDirs)) {
		foreach ($pluginDirs as $dir) {
			$include = AES_SITE_PATH.'/plugins/'.$dir.
			           '/AES_registerPlugin.inc.php';
			$ret = include($include);
			if ($ret === TRUE) {
				AES_print( sprintf(_("Plugin file ".
						"'%s/plugins/<b>%s</b>/AES_registerPlugin.inc.php' ".
						"<font color=\"red\">loaded</font>."),
						AES_SITE_PATH, $dir ) );
			}
			else {
				// this case should NOT happen, but ...
				AES_print( sprintf(_("Inclusion of plugin file '<b>%s</b>' ".
						"<font color=\"red\">failed</font>."),
						$include) );
			}
		}
	}
}

/*
 * Load plugins in AES main directory
 */
$pluginDirs =& _AES_getPluginsDirName(AES_INST_PATH.'/plugins');
//loads all plugins in "plugins" directoty; they must contain a file
// "PLUGIN_NAME/AES_registerPlugin.inc.php" otherwise it will be ignored
// and added to AES_ignoredPlugins !
foreach ($pluginDirs as $dir) {
	$include = AES_INST_PATH.'/plugins/'.$dir.'/AES_registerPlugin.inc.php';
	$ret = include $include;
	if ($ret === TRUE) {
		AES_print( sprintf(_("Plugin file ".
		           "'%s/plugins/<b>%s</b>/AES_registerPlugin.inc.php' ".
		           "<font color=\"red\">loaded</font>."),
		           AES_INST_PATH, $dir ) );
	}
	else {
		// this case should NOT happen, but ...
		AES_print( sprintf(_("Inclusion of plugin file '<b>%s</b>' ".
		           "<font color=\"red\">failed</font>."),
		           $include) );
	}
}
?>
