<?php
/* manageCallback.inc.php
 *
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * | AlberT-EasySite: Web Site Management System                          |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; either version 2 of the License.       |
 * +----------------------------------------------------------------------+
 * | Authors: Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: manageCallback.inc.php,v 1.3.2.2 2003/01/13 01:37:25 AlberT Exp $
 */


// callback definition
class AES_callback
{
	var $object    = NULL;  // 'object' to act upon
	var $functions = NULL;  // function to call
}

// Callback array for all 'objects'
$AES_callbackArray = array();

// Initialize callback array
for ($i = 0; $i < AES_MAX_CALLBACKS; ++$i) {
	$callback = new AES_callback;
	$callback->object = -1;
	$AES_callbackArray[$i] = $callback;
}

///////////////////////////////////////////////////////////////////////////////
//
// function AES_add_callback($object, $callback)
//
///////////////////////////////////////////////////////////////////////////////
//
// Adds a callback address
//
///////////////////////////////////////////////////////////////////////////////
//
// Parameter:
//  $object    - object to call function upon
//  $callbacks - functions to set for callback (array)
//
///////////////////////////////////////////////////////////////////////////////
//
// Return value:
//  FALSE - error
//  TRUE  - ok
//
///////////////////////////////////////////////////////////////////////////////
//
// Global references:
//  $AES_callback_array
//
///////////////////////////////////////////////////////////////////////////////
function AES_addCallback($objName, $callbacks)
{
	global $AES_callbackArray;

	// find empty slot for callback
	for ($i=0; $i < AES_MAX_CALLBACKS; ++$i) {
		$slot = $AES_callbackArray[$i];
		if ($slot->object == -1) {                  // free slot found...
			//... initialize it
			$slot->object = $objName;
			foreach ($callbacks as $callback) {     // add the callbacks
				$slot->functions[] = $callback;
			}
			// re-assign slot
			$AES_callbackArray[$i] = $slot;
			// return success
			return TRUE;
		}
	}
	// no free slot found, return error
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// function AES_removeCallback($object, $callback)
//
///////////////////////////////////////////////////////////////////////////////
//
// Removes a callback address
//
///////////////////////////////////////////////////////////////////////////////
//
// Parameter:
//  $object    - object to call function upon
//  $functions - function to set for callback
//
///////////////////////////////////////////////////////////////////////////////
//
// Return value:
//  FALSE - error
//  TRUE - ok
//
///////////////////////////////////////////////////////////////////////////////
//
// Global references:
//  $AES_callback_array
//
///////////////////////////////////////////////////////////////////////////////
function AES_removeCallback($object, $callback)
{
	global $AES_callbackArray;

	// find slot for callback
	for ($i = 0; $i < AES_MAX_CALLBACKS; ++$i) {
		$slot = $AES_callbackArray[$i];
		if (($slot->object == $object) && ($slot->functions == $callback)) {
			// slot found...
			//... kill it
			$slot->object = -1;
			unset($slot->functions);
			//unset($slot->order);
			// re-assign slot
			$AES_callbackArray[$i] = $slot;
			// return success
			return TRUE;
		}
	}
	// slot not found, return error
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
// function AES_doCallback($object)
//
///////////////////////////////////////////////////////////////////////////////
//
// Calls all callbacks for the given code.
//
///////////////////////////////////////////////////////////////////////////////
//
// Parameters:
//  $object - object to act upon
//
///////////////////////////////////////////////////////////////////////////////
//
// Globals referenced:
//  $AES_callbackArray
//
///////////////////////////////////////////////////////////////////////////////
function AES_doCallback($objName, $param)
{
	global $AES_callbackArray, $AES_objects, $AES_objInstanceCount;

	// search through whole callback array
	for ($i=0; $i<AES_MAX_CALLBACKS; ++$i) {
		// get this callback's info
		$slot = $AES_callbackArray[$i];
		// does it match the requested code ?
		if ($slot->object == $objName) {
			// object is matching, retrieve functions address
			$callbacks = $slot->functions;
			// call them
			foreach ($callbacks as $callback) {
				$class = "AES_$objName";
				if (empty($AES_objInstanceCount[$objName]) 
				    || !isset($AES_objInstanceCount[$objName])) {
					$AES_objInstanceCount[$objName] = 1;
				}
				if(!defined('AES_MAX_'.$objName.'_INSTANCES')) {
					define('AES_MAX_'.$objName.'_INSTANCES', 1);
				}
				$index = min($AES_objInstanceCount[$objName], 
				             constant('AES_MAX_'.$objName.'_INSTANCES'));
				$obj = $AES_objects[$objName][$index];
				if (class_exists($class) && is_object($obj)) {
					call_user_func_array(array(&$obj, $callback), $param);
					$AES_objInstanceCount[$objName] += 1;
				}
				else {
					AES_print(_("AES_doCalback: <font color=\"red\">ERROR!".
					            "</font> a callback is registred for object ".
					            "'<b>$object</b>', but class 'AES_$object' do ".
					            "NOT exists OR it is NOT inizialized by ".
					            "AES_registerPlugins() ..."), E_ERROR);
					return FALSE;
				}
			}
		}
	}
	return TRUE;
}
?>
