<?php
/*
    Name:       phpSecurityAdm
    Author:     Justin Koivisto (justin@koivi.com)
    Version:    3.0
    Date:       12/31/2002 4:49PM
    Copyright (C) 2001,2002

    This class was re-written from bottom-up to work with MetaBase.

    This program is free software; you can redistribute it and/or modify it under the terms of
    the GNU General Public License as published by the Free Software Foundation; either version
    2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details. (Found in the LICENSE file.)

    You should have received a copy of the GNU General Public License along with this program;
    if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA 02111-1307 USA
*/

//make sure this file is only included once
if(isset($PSA_CLASS_INC)) return;
$PSA_CLASS_INC=TRUE;

class phpSecurityAdm{
    var $db=NULL;                   // The Metabase object that is used to communicate with the database
    var $ERROR='';                  // The error message string identifiers for gettext to replace
    var $VERSION='3.0';             // The version of PSA
    var $HOST='';                   // The site name that PSA is working for
    var $PATH='';                   // The path to the PSA root directory
    var $LANGUAGE='en';             // The language to display messages and text in
    var $SITE_NAME='';              // The site name
    var $USER_ERROR_FUNCTION='';    // Name of user-defined error function.

    /**
    *   phpSecurityAdm
    *
    *   The class constructor.
    *
    *   @param  $PSA_SCR    Array(
    *                       'Host'          The database server's host name or address
    *                       'User'          The user to log into the database with
    *                       'Password'      The password to log into the datbase with
    *                       'IncludePath'   The system path where Metabase is located
    *                       'Type'          The type of database system (mysql,mssql,pgsql,etc) that is supported my MetaBase
    *                       'Database'      The name of the database to connect to
    *                       'Language'      PSA's default language to use
    *                       'PsaPath'       PSA's system path to the root of the distribution
    *                   )
    *   @result Creates a PSA object for use. $object->ERROR string is set according to errors that may have occurred.
    */
    function phpSecurityAdm($PSA_SCR){
        // set the PsaPath for the application
        if(isset($PSA_SCR['PsaPath']))
            $this->PATH=$PSA_SCR['PsaPath'];
        else
            $this->PATH=dirname(__FILE__).'/..';    // defaults to one directory back since this is in the include directory

        // the language to use (currently only english, so this doesn't do anything yet)
        if(isset($PSA_SCR['Language']))
            $this->LANGUAGE=$PSA_SCR['Language'];
        else
            $this->LANGUAGE='en';
        putenv('LANG='.$this->LANGUAGE);
        setlocale(LC_ALL,$this->LANGUAGE);

        // The site name stuff
        if(isset($PSA_SCR['SiteName'])){
            $this->SITE_NAME=$PSA_SCR['SiteName'];
        }else{
            $this->SITE_NAME=$_SERVER['HTTP_HOST'];
        }

        // include the necessary Metabase files
        require_once $PSA_SCR['IncludePath'].'/metabase_interface.php';
        require_once $PSA_SCR['IncludePath'].'/metabase_database.php';

        // create the Metabase object
        $this->ERROR = MetabaseSetupDatabaseObject($PSA_SCR,$this->db);
        if(!empty($this->ERROR))
            return FALSE;
        $this->db->setDatabase($PSA_SCR['Database']);
        return TRUE;
    }

    /**
    *   isLoggedIn
    *
    *   Checks to see if the user is logged in through the session variables.
    *   $result returns TRUE or FALSE depending on result
    */
    function isLoggedIn(){
        $this->ERROR='';
        // if session remote value doesn't equal the remote address, don't allow access.
        if(isset($_SESSION['PSA_psaun']) && isset($_SESSION['PSA_remote']) &&
                ($_SESSION['PSA_remote'] == $_SERVER['REMOTE_ADDR'])){
            return TRUE;
        }
        return FALSE;
    }

    /**
    *   Login
    *
    *   Performs the actual login procedure with all error checking.
    *
    *   @param  user    String with username
    *   @param  pass    String with password
    *   @result returns TRUE if successful login or FALSE if unsuccessful. Error message string is set on failure.
    */
    function Login($user='',$pass=''){
        $this->ERROR='';
        if($user==''){
            $this->ERROR=_("No username was provided.");
            return FALSE;
        }
        if($pass==''){
            $this->ERROR=sprintf(_("Bad password for user: <b>%s</b>"),$user);
            return FALSE;
        }

        // I want to make sure that there are no sneaks trying to pass SQL queries!
        // removing spaces will render things like "DELETE FROM" useless, so no need to do any more
        $user=str_replace(' ','',$user);
        // password is md5() encoded before comparison - no need to check value (might want spaces in password)

        // clear session variables
        $_SESSION['PSA_psaun']='';
        $_SESSION['PSA_remote']='';
        session_unset();
        
        // force a new, random session id to prevent malacious creation of one
        // this is mostly for those that decide not to use the custom session handlers in the include/session.php file
        session_id(md5(uniqid(rand(),1)));

        // check if username is in the system
        if($this->isSystemUser($user)){
            // get user info from the database
            $query="SELECT pass, hash, active, lockcount, connection FROM psa_users WHERE id=".
                $this->db->GetTextFieldValue($user);
            $result=$this->db->Query($query);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            $db_pass=$this->db->FetchResult($result,0,0);
            $db_hash=$this->db->FetchResult($result,0,1);
            $active=$this->db->FetchResult($result,0,2);
            $lockcount=$this->db->FetchResult($result,0,3)+1;
            $connection=explode(',',$this->db->FetchResult($result,0,4));

            // get the system lockout number
            $query="SELECT value FROM psa_settings WHERE name=".$this->db->GetTextFieldValue('lockcount');
            $result=$this->db->Query($query);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            $sys_lock=$this->db->FetchResult($result,0,0);
            if($lockcount>$sys_lock){
                $query="UPDATE psa_users SET updated=".$this->GetDateFieldValue(date('Y-m-d H:i:s')).
                    ", active=0, lockcount=0 WHERE id=".$this->GetTextFieldValue($user);
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
                $this->ERROR=_("Your account has either been locked or has not been activated");
                return FALSE;
            }

            // check if user account is active
            if(!$active){
                $this->ERROR=_("Your account has either been locked or has not been activated");
                return FALSE;
            }

            // check if connection is allowed
            if(!$this->checkConnection($user)){
                // connection not allowed!
                $query="UPDATE psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s')).", lockcount=".
                    $lockcount." WHERE id=".$this->db->GetTextFieldValue($user);
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
                $this->ERROR=sprintf(_("You are not allowed to login from the address <b>%s</b>"),$_SERVER['REMOTE_ADDR']);
                return FALSE;
            }

            // check the password
            $pass=trim($pass);
            if(strlen($pass)>0 && md5($pass.$db_hash)==$db_pass){
                $query="UPDATE psa_users SET updated=".$this->db->GetTextFieldValue(date('Y-m-d H:i:s')).
                    ", lockcount=0 WHERE id=".$this->db->GetTextFieldValue($user);
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
                $psaun=$user;
                $remote=$_SERVER['REMOTE_ADDR'];
                $_SESSION['PSA_psaun']=$psaun;
                $_SESSION['PSA_remote']=$remote;
                return TRUE;
            }else{
                $query="UPDATE psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s')).", lockcount='".$lockcount.
                    " WHERE id=".$this->db->GetTextFieldValue($user);
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
                $this->ERROR=sprintf(_("Bad password supplied for user <b>%s</b>"),$user);
                return FALSE;
            }
        }else{
            $this->ERROR=sprintf(_("The user <b>%s</b> does not exist on the system"),$user);
            return FALSE;
        }
    }

    /**
    *   Logout
    *
    *   Logs out the user and destroys the session.
    *   @result always returns TRUE
    */
    function Logout(){
        $this->ERROR='';
        if(isset($_SESSION['PSA_psaun'])){
            // clear session variables
            $_SESSION['PSA_psaun']='';
            $_SESSION['PSA_remote']='';
            session_unset();
            session_destroy();
        }
        return TRUE;
    }

    /**
    *   checkConnection
    *
    *   Check to see if the user is allowed to connect from the current address
    *
    *   @paran  $user   The username that is to log in
    *   @result returns TRUE or FALSE and sets error string appropriately
    */
    function checkConnection($user){
        $this->ERROR='';
        $SYSFLAG=FALSE;
        $ipsegs=explode('.',$_SERVER['REMOTE_ADDR']);
        $pattern='^('.$ipsegs[0].'){1,}\.('.$ipsegs[1].')+\.('.$ipsegs[2].')+\.('.$ipsegs[3].')+$';

        // check system permission first
        $query="SELECT value FROM psa_settings WHERE name=".$this->db->GetTextFieldValue('connection');
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $connection=$this->db->FetchResult($result,0,0);
        if($connection){
            // for each of the listed accepted address ranges, check if it matches
            $connections=explode(',',$connection);    // separate multiples
            while(list($x,$conn)=@each($connections)){
                $pattern='^'.str_replace('.','\.',trim($conn));
                if(preg_match($pattern,$_SERVER['REMOTE_ADDR'])){
                    $SYSFLAG=TRUE;
                }
            }
        }else{
            $SYSFLAG=TRUE;
        }

        if($SYSFLAG){
            $USERFLAG=FALSE;
            $query="SELECT connection FROM psa_users WHERE id=".$this->db->GetTextFieldValue($user);
            $result=$this->db->Query($query);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            $connection=$this->db->FetchResult($result,0,0);
            if($connection){
                // for each of the listed accepted address ranges, check if it matches
                $connections=explode(',',$connection);    // separate multiples
                while(list($x,$conn)=@each($connections)){
                    $pattern='^'.str_replace('.','\.',trim($conn));
                    if(preg_match($pattern,$_SERVER['REMOTE_ADDR'])){
                        $USERFLAG=TRUE;
                    }
                }
            }else{
                $USERFLAG=TRUE;
            }
        }
        return ($SYSFLAG && $USERFLAG);
    }

    /**
    *   isSystemUser
    *
    *   Check to see if username exists on system.
    *
    *   @param  user    The username to check for
    *   @result returns TRUE or FALSE and sets error string appropriately
    */
    function isSystemUser($user){
        $this->ERROR='';
        // don't even look up if not in valid format
        if(!$this->isValidUsername($user)){
            return FALSE;
        }
        $query="SELECT id FROM psa_users WHERE id=".$this->db->GetTextFieldValue($user);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        if($this->db->NumberOfRows($result)){
            return TRUE;
        }
        return FALSE;
    }

    /**
    *   isValidUsername
    *
    *   Check to see if the username is only alpha-numeric characters and is of the proper length according to the system settings.
    *
    *   @param  user    The username to check
    *   @result returns TRUE or FALSE and sets error string appropriately.
    */
    function isValidUsername($user){
        $this->ERROR='';
        $query="SELECT value FROM psa_settings WHERE name=".$this->db->GetTextFieldValue('max_username_length');
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $max_username_length=$this->db->FetchResult($result,0,0);

        $query="SELECT value FROM psa_settings WHERE name=".$this->db->GetTextFieldValue('min_username_length');
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $min_username_length=$this->db->FetchResult($result,0,0);

        $regex='/^[a-zA-Z0-9]+$/';
        $test=strlen($user);
        if($test > $max_username_length){
            $this->ERROR=sprintf(_("Username <b>%s</b> is too long. The username must be between %d and %d characters long."),
                $user,$min_username_length,$max_username_length);
            return FALSE;
        }
        if($test < $min_username_length){
            $this->ERROR=sprintf(_("Username <b>%s</b> is too short. The username must be between %d and %d characters long."),
                $user,$min_username_length,$max_username_length);
            return FALSE;
        }
        if(!preg_match($regex,$user)){
            $this->ERROR=sprintf(_("Username <b>%s</b> contains invalid characters. Only letters and numbers are allowed in a user name."),
                $user);
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   isValidPassword
    *
    *   Check to make sure the password is of proper length.
    *
    *   @param  pass    The password to check.
    *   @result returns TRUE or FALSE and sets error string appropriately.
    */
    function isValidPassword($pass){
        $this->ERROR='';
        $query="SELECT value FROM psa_settings WHERE name='max_password_length'";
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $max_password_length=$this->db->FetchResult($result,0,0);

        $query="SELECT value FROM psa_settings WHERE name='min_password_length'";
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $min_password_length=$this->db->FetchResult($result,0,0);

        $test=strlen($pass);
        if($test > $max_password_length){
            $this->ERROR=sprintf(_("The supplied password is too long. The password must be between %d and %d characters long."),
                $min_password_length,$max_password_length);
            return FALSE;
        }else if($test < $min_password_length){
            $this->ERROR=sprintf(_("The supplied password is too short. The password must be between %d and %d characters long."),
                $min_password_length,$max_password_length);
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   isAdmin
    *
    *   @param  user    username to lookup and test - check for administrative rights
    *   @result returns TRUE or FALSE and sets error string appropriately.
    */
    function isAdmin($user){
        $this->ERROR='';
        $query="SELECT groups FROM psa_users WHERE id=".$this->db->GetTextFieldValue($user);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $groups=explode(',',$this->db->FetchResult($result,0,0));

        foreach($groups as $gid){
            if($gid>0) {
                $query="SELECT admin FROM psa_profiles WHERE id=".$gid;
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
                if($this->db->FetchBooleanResult($result,0,0)){
                    return TRUE;
                }
            }
        }
        // if the function is still running at this point, the result must be false
        return FALSE;
    }

    /**
    *   hasRights
    *
    *   Check to see if the user has rights to view this page.
    *
    *   @param  url The URL to check ($_SERVER['PHP_SELF'] may be good)
    *   @param  url2    Optional URL to check ($_SERVER['REQUEST_URI'] may be good)
    *   @result returns TRUE or FALSE and sets error string appropriately.
    */
    function hasRights($url,$url2=''){
        $this->ERROR='';
        // Make sure that the database will contain the full URI for the page
        $url=ereg_replace('(\?|&)'.session_name().'=[a-zA-Z0-9]{32}','',$url);
        $url2=ereg_replace('(\?|&)'.session_name().'=[a-zA-Z0-9]{32}','',$url2);
        $query="SELECT id FROM psa_pages WHERE page=".$this->db->GetTextFieldValue($url).
            " OR page=".$this->db->GetTextFieldValue($url2);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $pageid=$this->db->FetchResult($result,0,0);

        $query="SELECT groups FROM psa_users WHERE id=".$this->db->GetTextFieldValue($_SESSION['PSA_psaun']);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $groups=explode(',',$this->db->FetchResult($result,0,0,0));

        $tmp=$this->getProfiles();
        while(list($k,$gid)=@each($groups)){
            $rightsar=$this->getProfile($gid);
            if(isset($rightsar[$pageid]))
                return TRUE;
        }
        $this->ERROR=_("You do not have access rights to this content");
        return FALSE;
    }

    /**
    *   createRights
    *
    *   Takes an array and creates a field value for rights (users/profiles). Only adds rights to pages that are in the database.
    *
    *   @param  ar  array[page id]=access right
    *   @result FALSE or Returns a serialized array for storing in the rights field of the psa_users table and sets error string.
    */
    function createRights($ar){
        $this->ERROR='';
        $i=0; $retarr=array();
        $pages=$this->getPages();
        if($pages===FALSE){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        while(list($key,$val)=@each($ar)){
            if(isset($pages[$key]))
                $retarr[$key]=$val;
            $i++;
        }
        return serialize($retarr);
    }

    /**
    *   deleteUser
    *
    *   Remove a user from the system.
    *
    *   @param  user    The username you wish to delete from the database.
    *   @result returns TRUE or FALSE and sets error string.
    */
    function deleteUser($user){
        $this->ERROR='';
        $query="DELETE FROM psa_users WHERE id=".$this->db->GetTextFieldValue($user);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   addProfile
    *
    *   Takes an array and creates a system user profile based on it.
    *
    *   param: ar array[field]=bool
    *   @result returns TRUE or FALSE and sets error string.
    */
    function addProfile($ar){
        $this->ERROR='';
        $name=$ar['PSA_name'];
        if(isset($ar['PSA_admin']))
            $admin=1;
        else
            $admin=0;
        unset($ar['PSA_admin']);
        unset($ar['PSA_name']);
        $rights=$this->createRights($ar);
        $result=$this->db->GetSequenceNextValue('psa_profiles_id',$id);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $query="INSERT INTO psa_profiles SET id=".$id.", name=".$this->db->GetTextFieldValue($name).", value=".
            $this->db->GetTextFieldValue($rights).", admin=".$this->db->GetBooleanFieldValue($admin);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   editProfile
    *
    *   Takes an array and updates a profile based on it.
    *
    *   @param  ar array[field]=bool
    *   @result returns TRUE or FALSE and sets error string.
    */
    function editProfile($ar){
        $this->ERROR='';
        $id=$ar['PSA_id'];
        $name=$ar['PSA_name'];
        if(isset($ar['PSA_admin']))
            $admin=$ar['PSA_admin'];
        else
            $admin=0;
        $ar=$this->createRights($ar);
        $query="UPDATE psa_profiles SET name=".$this->db->GetTextFieldValue($name).", value=".
            $this->db->GetTextFieldValue($ar['PSA_value']).",admin=".$this->db->GetBooleanFieldValue($admin)." WHERE id=".$id;
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   deleteProfile
    *
    *   Takes a profile id and deletes the profile.
    *
    *   @param  id  The profile's database row id.
    *   @result returns TRUE or FALSE and sets error string.
    */
    function deleteProfile($id){
        $this->ERROR='';
        $query="DELETE FROM psa_profiles WHERE id=".$id;
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   addPage
    *
    *   Adds a page to the database to allow restrictions.
    *
    *   @param  title   The name you want to see the page displayed as in the rights list.
    *   @param  page    The REQUEST_URI of the page.
    *   @result returns TRUE or FALSE and sets the error string.
    */
    function addPage($title,$page){
        $this->ERROR='';
        $query="SELECT id FROM psa_pages WHERE title=".$this->db->GetTextFieldValue($title)." OR page=".
            $this->db->GetTextFieldValue($page);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        if($this->db->NumberOfRows($result)){
            return FALSE;
        }else{
            $result=$this->db->GetSequenceNextValue('psa_pages_id',$id);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            $query="INSERT INTO psa_pages SET id=".$id.", title=".$this->db->GetTextFieldValue($title).", page=".
                $this->db->GetTextFieldValue($page);
            $result=$this->db->Query($query);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            return TRUE;
        }
    }

    /**
    *   editPage
    *
    *   Edits a page in the database info that is sent via array.
    *
    *   @param  ar  array[field]=value
    *   @result returns TRUE or FALSE and sets error string.
    */
    function editPage($ar){
        $this->ERROR='';
        $query="UPDATE psa_pages SET page=".$this->db->GetTextFieldValue($ar['PSA_page']).", title=".
        $this->db->GetTextFieldValue($ar['PSA_title'])." WHERE id=".$ar['PSA_id'];
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   deletePage
    *
    *   Deletes a page from the database. If this is done nobody can view that page unless the include line
    *   for _restrict.php is removed from the file on the server.
    *
    *   @param  id  The database row id of the page to delete.
    *   @result returns TRUE or FALSE and sets error string.
    */
    function deletePage($id){
        $this->ERROR='';
        $query="DELETE FROM psa_pages WHERE id=".$id;
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   getPages
    *
    *   Get an array of all the page ids and titles.
    *
    *   @result returns an array in the format array[pageid]=title
    */
    function getPages(){
        $this->ERROR='';
        $pages=array();
        $query='SELECT id, title FROM psa_pages ORDER BY title';
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        for($i=0;$i<$this->db->NumberOfRows($result);$i++){
            $id=$this->db->FetchResult($result,$i,0);
            $title=$this->db->FetchResult($result,$i,1);
            $pages[$id]=$title;
        }
        return $pages;
    }

    /**
    *   getPage
    *
    *   Get an array of the page values from the table.
    *
    *   @param  id  The page id in the database
    *   @result returns array[field]=value or FALSE on ERROR
    */
    function getPage($id){
        $this->ERROR='';
        $query="SELECT page, title FROM psa_pages WHERE id=".$id;
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $retarr['id']=$id;
        $retarr['page']=$this->db->FetchResult($result,0,0);
        $retarr['title']=$this->db->FetchResult($result,0,1);
        return $retarr;
    }

    /**
    *   getProfiles
    *
    *   Get an array of all the profile ids and names in the system.
    *
    *   @result returns array[id]=name else FALSE on error.
    */
    function getProfiles(){
        $this->ERROR='';
        $profiles=array();
        $query='SELECT id, name FROM psa_profiles';
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        for($i=0;$i<$this->db->NumberOfRows($result);$i++){
            $id=$this->db->FetchResult($result,$i,0);
            $name=$this->db->FetchResult($result,$i,1);
            $profiles[$id]=$name;
        }
        ksort($profiles);
        return $profiles;
    }

    /**
    *   getProfile
    *
    *   Get the profile information from the database.
    *
    *   @param  id  the profile's id in the database
    *   @result returns FALSE if profile doesn't extist or Array[pageid]=$access_flag as well as
    *           array['name']=name and array['admin']=T/F if profile has been set else returns
    *           $array['name']=name if it is an empty profile.
    */
    function getProfile($id){
        $this->ERROR='';
        $query="SELECT value, name, admin FROM psa_profiles WHERE id=".$id;
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        if($this->db->NumberOfRows($result)){
            $rights=@unserialize($this->db->FetchResult($result,0,0));
            $rights['rights']=$this->db->FetchResult($result,0,0);
            $rights['name']=$this->db->FetchResult($result,0,1);
            $rights['admin']=$this->db->FetchBooleanResult($result,0,2);
            return $rights;
        }else{
            $this->ERROR=sprintf(_("The profile with id <b>%d</b> does not exist."),$id);
            return FALSE;
        }
    }

    /**
    *   getUsers
    *
    *   Get array of users.
    *
    *   @result returns array(name,email,phone,connection,admin,active,lanme,fname,mname,groups)
    *   or FALSE on error - sets error string
    */
    function getUsers(){
        $this->ERROR='';
        $users=array(); $groups=array();
        $query='SELECT id, admin FROM psa_profiles ORDER BY id';
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        for($i=0;$i<$this->db->NumberOfRows($result);$i++){
            $idx=$this->db->FetchResult($result,0,0);
            $val=$this->db->FetchBooleanResult($result,0,1);
            $groups[$idx]=$val;
        }

        $query='SELECT id, fname, lname, mname, email, groups, phone, connection, active FROM psa_users';
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }

        for($i=0;$i<$this->db->NumberOfRows($result);$i++){
            $id=$this->db->FetchResult($result,$i,0);
            $fname=$this->db->FetchResult($result,$i,1);
            $lname=$this->db->FetchResult($result,$i,2);
            $mname=$this->db->FetchResult($result,$i,3);
            $email=$this->db->FetchResult($result,$i,4);
            $phone=$this->db->FetchResult($result,$i,6);
            $groups_tmp=$this->db->FetchResult($result,$i,5);
            $connection=$this->db->FetchResult($result,$i,7);
            $active=$this->db->FetchBooleanResult($result,$i,8);
            $name=$lname.', '.$fname;

            $tmp=split(',',$groups_tmp);
            $admin=0;
            foreach($tmp as $gid){
                if(isset($groups[$gid])){
                    $admin=1;
                    break;
                }
            }

            $users[$id]=array('id'=>$id,'name'=>$name,'email'=>$email,'phone'=>$phone,
                'connection'=>$connection,'admin'=>$admin,'active'=>$active,
                'lname'=>$lname,'fname'=>$fname,'mname'=>$mname,'groups'=>join(',',$tmp));
        }
        return $users;
    }

    /**
    *   loginForm
    *
    *   Displays a login form that posts to page that called function. This only displays the form elemnt, so be sure to include the
    *   rest of the HTML for the page or you may not get the expected results with some browsers.
    *
    *   @param  custom  Optional filename to include as the login form.
    */
    function loginForm($custom=''){
        $this->ERROR='';
        if($custom){
            include $custom;
            return TRUE;
        }else{
            // check if REQUEST_URI is set, if not, use PHP_SELF to post the form to
            if(isset($_SERVER['REQUEST_URI'])){
                echo '          <form method="POST" action="'.$_SERVER['REQUEST_URI'].'">'."\n";
                echo '           <input type="hidden" name="PSA_requested_uri" value="'.$_SERVER['REQUEST_URI'].'">'."\n";
            }else{
                echo '          <form method="POST" action="'.$_SERVER['PHP_SELF'].'">'."\n";
                echo '           <input type="hidden" name="PSA_requested_uri" value="'.$_SERVER['PHP_SELF'].'">'."\n";
            }
            while(list($k,$v)=@each($_POST)){
                echo '<input type="hidden" name="PSA_'.$k.'" value="'.urlencode($v).'">'."\n";
            }
?>
           <table cellpadding="0" cellspacing="3" border="0">
            <tr>
             <td><b><?php echo _("Username") ?></b></td>
             <td><input type="text" size="30" name="PSA_user"></td>
            </tr>
            <tr>
             <td><b<?php echo _("Password") ?></b></td>
             <td><input type="password" size="30" name="PSA_pass"></td>
            </tr>
           </table>
           <input type="hidden" name="PSA_language" value="<?php echo $this->LANGUAGE ?>">
           <input type="submit" name="PSA_LOGIN" value="<?php echo _("Login") ?>">
          </form>
<?php
            return TRUE;
        }
    }

    /**
    *   editUser
    *
    *   Updates the user information in the database based on the passed values.
    *
    *   @param  ar  Array of key/value pairs to set ($_POST works well with this if you have the correct field names).
    *   @result Returns TRUE or FALSE and set error string accordingly.
    */
    function editUser($ar){
        $this->ERROR='';
        $query="UPDATE psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s'));
        if(isset($ar['PSA_pass']) && $this->isValidPassword($ar['PSA_pass']) && isset($ar['PSA_pass']) && $ar['PSA_pass']==$ar['PSA_pass2']){
            $this->changePassword($ar['PSA_id'],$ar['PSA_pass']);
        }
        while(list($field,$value)=@each($ar)){
            if($field=='lname' || $field=='fname' || $field=='mname' ||
                    $field=='email' || $field=='phone' || $field=='connection'){    // text value fields
                $query.=", ".$field."=".$this->db->getTextFieldValue($value);
            }
        }
        if(isset($ar['PSA_active'])){  // boolean value field
            $query.=", active=".$this->db->getBooleanFieldValue($ar['PSA_active']);
        }

        $query.=" WHERE id=".$this->db->GetTextFieldValue($ar['PSA_id']);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   activateUser
    *
    *   Activates or Deactivates a user in the database according to passed parameter.
    *
    *   @param  $user   the user to activate/deactivate
    *   @param  $active Pass as 0 to deactivate a user.
    *
    *   result  returns TRUE or FALSE and sets error string.
    */
    function activateUser($user,$active=1){
        $this->ERROR='';
        $query="UPDATE psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s')).", active=".
            $this->db->GetBooleanFieldValue($active)." WHERE id=".$this->db->GetTextFieldValue($user);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   changeProfile
    *
    *   Takes single array $ar to change a user's access rights profile ($_POST works well with this if you have the correct field names).
    *
    *   @param  ar  An array in the form array('id' => username, 'gids' => array of profile ids to apply).
    *   @result Returns TRUE or FALSE and sets error string.
    */
    function changeProfile($ar){
        $this->ERROR='';
        $id=$ar['PSA_id'];
        $tmp=$ar['PSA_gids'];
        foreach($ar as $key => $value){
            if($key==$value){
                $tmp[]=$ar[$key];
            }
        }
        $groups=join(',',$tmp);

        $query="UPDATE psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s')).", groups=".
            $this->db->GetTextFieldValue($groups)." WHERE id=".$this->db->GetTextFieldValue($id);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   addUser
    *
    *   Takes single array $ar to create a new user for the system.
    *
    *   @param  ar  Array in [field]=value pairs.
    *   @result Returns TRUE or FALSE and sets error message.
    */
    function addUser($ar){
        $this->ERROR='';
        $users=$this->getUsers();
        if(isset($users[$ar['PSA_id']])){
            $this->ERROR=sprintf(_("The username <b>%s</b> alread exists on the system."),$ar['PSA_id']);
            return FALSE;
        }
        $query="INSERT INTO psa_users SET updated=".$this->db->GetDateFieldValue(date('Y-m-d H:i:s'));

        $HASH_VAR=date('YmdHis').$_SERVER['REMOTE_ADDR'];
        $hash=md5($HASH_VAR);
        $password=md5($ar['PSA_pass'].$hash);

        if(isset($ar['PSA_gids'])){
            $groups=join(',',$ar['PSA_gids']);
        }else{
            $groups='';
        }

        $query.=", lname=".$this->db->GetTextFieldValue($ar['PSA_lname']).
            ", fname=".$this->db->GetTextFieldValue($ar['PSA_fname']).
            ", mname=".$this->db->GetTextFieldValue($ar['PSA_mname']).
            ", email=".$this->db->GetTextFieldValue($ar['PSA_email']).
            ", phone=".$this->db->GetTextFieldValue($ar['PSA_phone']).
            ", id=".$this->db->GetTextFieldValue($ar['PSA_id']).
            ", pass=".$this->db->GetTextFieldValue($password).
            ", hash=".$this->db->GetTextFieldValue($hash).
            ", groups=".$this->db->GetTextFieldValue($groups).
            ", active=".$this->db->GetBooleanFieldValue(1);
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        return TRUE;
    }

    /**
    *   changePassword
    *
    *   Updates the user's password in the database. The logged in user must either be the same as the
    *   requested change or someone with administrative rights to the system.
    *
    *   @param  user    The username to change the password for.
    *   @param  pass    The new password.
    *   @result Returns TRUE or FALSE and sets error string.
    */
    function changePassword($user,$pass){
        $this->ERROR='';
        if($_SESSION['PSA_psaun']==$user || $this->isAdmin($_SESSION['PSA_psaun'])){
            $HASH_VAR=date('YmdHis').$_SERVER['REMOTE_ADDR'];
            $hash=md5($HASH_VAR);
            $password=md5($pass.$hash);
            $query="UPDATE psa_users SET updated=".$hits->db->GetDateFieldValue(date('Y-m-d H:i:s')).", pass=".
                $this->db->GetTextFieldValue($password).", hash=".$this->db->GetTextFieldValue($hash).
                ", lockcount=0, active=".$this->db->GetBooleanFieldValue(1)." WHERE id=".$this->db->GetTextFieldValue($user);
            $result=$this->db->Query($query);
            if($result==0){
                $this->ERROR=$this->db->Error();
                return FALSE;
            }
            return TRUE;
        }else{
            $this->ERROR=_("You do not have rights to change this user's password.");
            return FALSE;
        }
    }

    /**
    *   updateSettings
    *
    *   Allows a user to change the security system settings.
    *
    *   @param  ar  Array containing field/value pairs for the settings to be updated.
    *   @result Returns TRUE or FALSE and sets the error string
    */
    function updateSettings($ar){
        while(list($name,$val)=@each($ar)){
            if($name!='submit'){
                $query="UPDATE psa_settings SET value='".$val."' WHERE name='".$name."'";
                $result=$this->db->Query($query);
                if($result==0){
                    $this->ERROR=$this->db->Error();
                    return FALSE;
                }
            }
        }
    }

    /**
    *   getSettings
    *
    *   Get the values for each of the system settings.
    *
    *   @result Returns an field/value paired array or FALSE and sets error string.
    */
    function getSettings(){
        $this->ERROR='';
        $retVal=array();
        $query='SELECT name, value FROM psa_settings';
        $result=$this->db->Query($query);
        if($result==0){
            $this->ERROR=$this->db->Error();
            return FALSE;
        }
        for($i=0;$i<$this->db->NumberOfRows($result);$i++){
            $retVal[$this->db->FetchResult($result,$i,0)]=$this->db->FetchResult($result,$i,1);
        }
        return $retVal;
    }

    /**
    *   setErrorFunction
    *
    *   Returns the class's error string or calls a custom function.
    *
    *   @param  $func   The name of the function to use.
    *   @result Returns TRUE and sets the USER_ERROR_FUNCTION name.
    */
    function setErrorFunction($func){
        $this->USER_ERROR_FUNCTION=$func;
        return TRUE;
    }

    /**
    *   Error
    *
    *   Returns the class's error string or calls a custom function. This is here for customization reasons.
    *
    *   @param  $str    The string to output as error.
    *   @param  $level  The error message level.
    *   @result Returns the error message.
    */
    function Error($str='',$level=0){
        if(empty($str))
            $str=$this->ERROR;
        if(function_exists($this->USER_ERROR_FUNCTION))
            return call_user_func($this->USER_ERROR_FUNCTION,$str,$level);
        return $str;
    }

} // end class

?>
