<?php
/*
    This file contains all the function definitions for the custom session
    handler that uses an abstracted database using Metabase.
*/

if (isset($PSA_METABASE_SESS_HANDLERS_INC)) return TRUE;
$PSA_METABASE_SESS_HANDLERS_INC=TRUE;

    // let the server know that you want to set your session handling yourself.
    // The manual is kind of funny with this. According to the ini_set page, this shouldn't
    // work, but it does (and a user has posted that fact). However, I've also noticed this
    // script working with session.save_handler = files. According to the
    // session_set_save_handler page, it shouldn't - I added a note on this.
    ini_set('session.save_handler','user');

    // I don't like the idea of having stale sessions around. Having them removed may even
    // enhance performance on the database table if it starts getting large. However, I don't
    // think it is necessary to run it at 100 - unless you are _very_ paranoid.
    ini_set('session.gc_probability','100');

    // config file holds connection data
    require_once dirname(__FILE__).'/config.php';
    
    // The path to Metabase was defined in the config.php file for PSA
    require_once $PSA_SCR['IncludePath'].'/metabase_interface.php';
    require_once $PSA_SCR['IncludePath'].'/metabase_database.php';

    $PSA_SESS_SQL=FALSE;            // MySQL object to be used by sessions

    // How long the sessions last. Defaults to the value in the php.ini file.
    $PSA_SESS_LIFE=ini_get('session.gc_maxlifetime');

    function sess_open(){
        global $PSA_SESS_SQL, $PSA_SCR;

        // create the Metabase object
        $ERROR = MetabaseSetupDatabaseObject($PSA_SCR,$PSA_SESS_SQL);
        if(!empty($ERROR))
            return FALSE;
        $PSA_SESS_SQL->setDatabase($PSA_SCR['Database']);
        return TRUE;
    }

    function sess_close(){
        return TRUE;
    }

    function sess_read($key){
        global $PSA_SESS_SQL;

        $query="select value from sessions where sesskey=".$PSA_SESS_SQL->GetTextFieldValue($key)." and expiry > ".time();
        $result=$PSA_SESS_SQL->Query($query);
        if($result==0){
            return '';
        }
        return $PSA_SESS_SQL->FetchResult($result,0,0);
    }

    function sess_write($key,$val){
        global $PSA_SESS_SQL,$PSA_SESS_LIFE;

        // Calculate the session end time
        $expiry=time()+$PSA_SESS_LIFE;
        $value=addslashes($val);
        $query="select sesskey from sessions where sesskey=".$PSA_SESS_SQL->GetTextFieldValue($key);
        $result=$PSA_SESS_SQL->Query($query);
        if($result==0){
            return '';
        }
        $sesskey=$PSA_SESS_SQL->FetchResult($result,0,0);

        if($sesskey){
            // if the session exists, update it
            $query="update sessions set expiry=".$expiry.", value=".$PSA_SESS_SQL->GetTextFieldValue($value).
                " where sesskey=".$PSA_SESS_SQL->GetTextFieldValue($key)." and expiry > ".time();
            $result=$PSA_SESS_SQL->Query($query);
            if($result==0){
                return '';
            }
        }else{
            // if the session doesn't exist, create it
            $query="insert into sessions set sesskey=".$PSA_SESS_SQL->GetTextFieldValue($key).", expiry=".$expiry.
                ", value=".$PSA_SESS_SQL->GetTextFieldValue($value);
            $result=$PSA_SESS_SQL->Query($query);
            if($result==0){
                return '';
            }
        }
        return TRUE;
    }

    function sess_destroy($sess_id){
        global $PSA_SESS_SQL;

        // delete the existing session
        $query="delete from sessions where sesskey=".$PSA_SESS_SQL->GetTextFieldValue($sess_id);
        $result=$PSA_SESS_SQL->Query($query);
        if($result==0){
            return '';
        }
        return TRUE;
    }

    function sess_gc(){
        global $PSA_SESS_SQL;

        // delete all expired sessions
        $query="delete from sessions where expiry < ".time();
        $result=$PSA_SESS_SQL->Query($query);
        if($result==0){
            return '';
        }

        $result=$PSA_SESS_SQL->AffectedRows($retVal);
        if($result==0){
            return '';
        }
        return $retVal;
    }

    session_set_save_handler('sess_open','sess_close','sess_read','sess_write','sess_destroy','sess_gc');
?>
