<?php
/*
 * Session Management for PHP3
 *
 * Copyright (c) 1998,1999 NetUSE GmbH
 *                    Boris Erdmann, Kristian Koehntopp
 *
 * Derived from db_mysql.inc by Sascha Schumann <sascha@schumann.cx>
 *
 * $Id: db_sql.inc,v 1.1 2002/06/22 12:49:30 AlberT Exp $
 *
 */ 

class DB_Sql {

  /* public: connection parameters */
  var $Host     = "";
  var $Database = "";

  /* public: configuration parameters */
  var $Auto_Free     = 0;     ## Set this to 1 for automatic msql_free_result()
  var $Debug         = 0;     ## Set this to 1 for debugging messages.
  var $Halt_On_Error = "yes"; ## "yes", "report" or "no"
  var $Seq_Table     = "db_sequence";

  /* public: result array and current row number */
  var $Record   = array();
  var $Row;

  /* public: current error number and error text */
  var $Error    = "";
  
  /* public: this is an api revision, not a CVS revision */
  var $type     = "msql";
  var $revision = "1.2";
  
  /* private: link and query handles */
  var $Link_ID  = 0;
  var $Query_ID = 0;

  /* public: constructor */
  function DB_Sql($query = "") {
    $this->query($query);
  }

  /* public: some trivial reporting */
  function link_id() {
    return $this->Link_ID;
  }

  function query_id() {
    return $this->Query_ID;
  }

  /* public: connection management */
  function connect($Database = "", $Host = "") {
    /* Handle defaults */
    if ("" == $Database)
      $Database = $this->Database;
    
    if ("" == $Host)
      $Host = $this->Host;
  
    /* establish connection, select database */
    if ( 0 == $this->Link_ID ) {
      // Check for local connect
      $this->Link_ID = empty($Host)? 
                         $this->Link_ID=msql_pconnect():
                         $this->Link_ID=msql_pconnect($Host);
    }
    
    // Still not connected? Raise error.
    if ( 0 == $this->Link_ID ) {
      $this->halt("pconnect($Host) failed.");
      return 0;
    }

    // Select current database
	  if (!@msql_select_db($Database, $this->Link_ID)) {
      $this->halt("cannot use database ".$Database);
    }
    
    return $this->Link_ID;
  }
  
  /* public: discard the query result */
  function free() {
    @msql_free_result($this->Query_ID);
    $this->Query_ID = 0;
  }

  /* public: perform a query */  
  function query($Query_String) {
    /* No empty queries, please. */
    if ($Query_String == "")
      return 0;
    
    if (!$this->connect())
      return 0; /* we already complained in connect() about that. */
    
    if ($this->Query_ID)
      $this->free(); /* discard previous result */
    
    if ($this->Debug)
      printf("Debug: query = %s<br>\n", $Query_String);

    $this->Query_ID = @msql_query($Query_String,$this->Link_ID);
    $this->Row   = 0;
    $this->Error = msql_error();
    if (!$this->Query_ID) {
      $this->halt("Invalid SQL: ".$Query_String);
    }

    /* will return false if query failed. This is okay with us. */
    return $this->Query_ID;
  }

  /* public: walk result set */
  function next_record() {
    if (!$this->Query_Id) {
      $this->halt("next_record called with no query pending.");
      return 0;
    }
    
    $this->Record = @msql_fetch_array($this->Query_ID);
    $this->Row   += 1;
    $this->Error = msql_error();

    $stat = is_array($this->Record);
    if (!$stat && $this->Auto_Free) {
      $this->free();
    }
    return $stat;
  }

  /* public: position in result set */
  function seek($pos = 0) {
    $status = @msql_data_seek($this->Query_ID, $pos);
    if ($status)
      $this->Row = $pos;
    else {
      $this->halt("seek($pos) failed: result has ".$this->num_rows()." rows.");
      
      /* half assed attempt to save the day,
       * but do not consider this documented or even
       * desireable behaviour.
       */
      @msql_data_seek($this->Query_ID, $this->num_rows());
      $this->Row = $this->num_rows();
      return 0
    }
    
    return 1;
  }

  /* public: table locking */
  function lock($table, $mode="write") {
    $this->halt("Does msql have table locking functions?");
    return false;
  }
  
  function unlock() {
    $this->halt("Does msql have table locking functions?");
  }

  /* public: evaluate the result (size, width) */
  function affected_rows() {
	  return @msql_affected_rows($this->Query_ID);
  }

  function num_rows() {
    return @msql_num_rows($this->Query_ID);
  }

  function num_fields() {
    return @msql_num_fields($this->Query_ID);
  }

  function nf() {
    return $this->num_rows();
  }

  function np() {
    print $this->num_rows();
  }

  function f($Name) {
    return $this->Record[$Name];
  }

  function p($Name) {
    print $this->Record[$Name];
  }

  /* public: sequence numbers */
  /* This will not work unless somebody implements locking functions. */
  function nextid($seq_name) {
    $this->connect();

    if ($this->lock($this->Seq_Table)) {
      /* get sequence number (locked) and increment */
      $q  = sprintf("select nextid from %s where seq_name = '%s'",
                $this->Seq_Table,
                $seq_name);
      $id  = @msql_query($q, $this->Link_ID);
      $res = @msql_fetch_array($id);

      /* No current value, make one */
      if (!is_array($res)) {
        $currentid = 0;
        $q = sprintf("insert into %s values('%s', %s)",
                 $this->Seq_Table,
                 $seq_name,
                 $currentid);
        $id = @msql_query($q, $this->Link_ID);
      } else {
        $currentid = $res["nextid"];
      }
      $nextid = $currentid + 1;
      $q = sprintf("update %s set nextid = '%s' where seq_name = '%s'",
               $this->Seq_Table,
               $nextid,
               $seq_name);
      $id = @msql_query($q, $this->Link_ID);
      $this->unlock();
    } else {
      $this->halt("cannot lock ".$this->Seq_Table." - has it been created?");
      return 0;
    }
    return $nextid;
  }
  
  /* public: return table metadata */
  function metadata($table = "", $full = false) {
    $count = 0;
    $id    = 0;
    $res   = array();

    /* if no $table specified, we assume to be working with a query result */
    if ($table) {
      $this->connect();
      $id = @msql_list_fields($this->Database, $table);
      if (!$id) {
        $this->halt("Metadata query failed.");
        return false;
      }
    } else {
      $id = $this->Query_ID;
      if (!$id) {
        $this->halt("No query specified.");
        return false;
      }
    }
    
    $count = @msql_num_fields($id);
    
    if (!$full) {
      for ($i=0; $i<$count; $i++) {
        $res[$i]["table"] = msql_fieldtable ($id, $i);
        $res[$i]["name"]  = msql_fieldname  ($id, $i);
        $res[$i]["type"]  = msql_fieldtype  ($id, $i);
        $res[$i]["len"]   = msql_fieldlen   ($id, $i);
        $res[$i]["flags"] = msql_fieldflags ($id, $i);
      }
    } else {
      $res["num_fields"]= $count;

      for ($i=0; $i<$count; $i++) {
        $res[$i]["table"] = msql_fieldtable ($id, $i);
        $res[$i]["name"]  = msql_fieldname  ($id, $i);
        $res[$i]["type"]  = msql_fieldtype  ($id, $i);
        $res[$i]["len"]   = msql_fieldlen   ($id, $i);
        $res[$i]["flags"] = msql_fieldflags ($id, $i);
        $res["meta"][$res[$i]["name"]] = $i;
      }
    }

    /* we must not free foreign query results */
    if ($table)
      @msql_free_result($id);

    return $res;
  }

  /* public: find available table names */
  function table_names() {
    $this->connect();
    $h = @msql_query("show tables", $this->Link_ID);
    $i = 0;
    while ($info = @msql_fetch_row($h)) {
      $return[$i]["table_name"]      = $info[0];
      $return[$i]["tablespace_name"] = $this->Database;
      $return[$i]["database"]        = $this->Database;
      $i++;
    }
    
    @msql_free_result($h);
    return $return;
  }

  /* private: error handling */
  function halt($msg) {
    $this->Error = @msql_error($this->Link_ID);
    if ($this->Halt_On_Error == "no")
      return;
    
    $this->haltmsg($msg);
    
    if ($this->Halt_On_Error != "report")
      die("Session halted.");
  }
  
  function haltmsg($msg) {
    printf("</td></tr></table><b>Database error:</b> %s<br>\n", $msg);
    printf("<b>MSQL Error</b>: %s<br>\n", $this->Error);
  }
  
}
?>
