<?php
/*
 * Session Management for PHP3
 *
 * (C) Copyright 1998 Cameron Taggart (cameront@wolfenet.com)
 *        Modified by Guarneri carmelo (carmelo@melting-soft.com)
 *        Modified by Cameron Just     (C.Just@its.uq.edu.au) 
 *
 * $Id: db_sql.inc,v 1.1 2002/06/22 12:49:30 AlberT Exp $
 */ 

class DB_Sql {

  /* public: connection parameters */
  var $Host     = "";
  var $Database = "";
  var $User     = "";
  var $Password = "";

  /* public configuration parameters */
  var $Auto_Free = 0;     ## set this to 1 to automatically free results.
  var $Debug     = 0;     ## set this to 1 for debug output.
  var $Halt_On_Error = "yes";

  /* public: result array and current row number */
  var $Record   = array();
  var $Row      = 0;

  /* public: current error number and error text */
  var $Errno    = 0;
  var $Error    = "";

  /* public: this is an api revision, not a CVS revision. */
  var $type     = "mssql";
  var $revision = "1.2";
  
  /* private: link and query handles */
  var $Link_ID  = 0;
  var $Query_ID = 0;


  /* public: constructor */
  function DB_Sql($query = "") {
    $this->query($query);
  }

  /* public: some trivial reporting */
  function link_id() {
    return $this->Link_ID;
  }

  function query_id() {
    return $this->Query_ID;
  }

  /* public: connection management */
  function connect($Database = "", $Host = "", $User = "", $Password = "") {
    /* Handle defaults */
    if ("" == $Database)
      $Database = $this->Database;
    if ("" == $Host)
      $Host     = $this->Host;
    if ("" == $User)
      $User     = $this->User;
    if ("" == $Password)
      $Password = $this->Password;

    /* establish connection, select database */
    if ( 0 == $this->Link_ID ) {

      $this->Link_ID=mssql_pconnect($Host, $User, $Password);
      if (!$this->Link_ID) {
        $this->halt("pconnect($Host, $User, \$Password) failed.");
        return 0;
      }
      
      if (!@mssql_select_db($Database, $Link_ID)) {
        $this->halt("cannot select database $Database");
        return 0;
      }
    }
    
    return $this->Link_ID;
  }

  /* public: discard the query result */
  function free() {
    @mssql_free_result($this->Query_ID);
    $this->Query_ID = 0;
  }

  /* public: perform a query */
  function query($Query_String) {
    /* No empty queries, please! */
    if ($Query_String == "")
      return 0;
    
    if (!$this->connect())
      return 0; /* we already complained in connect() about that. */

    /* discard old result */
    if ($this->Query_ID)
      $this->free();

    if ($this->Debug)
      printf("Debug: query = %s<br>\n", $Query_String);
    
    $this->Query_ID = @mssql_query($Query_String, $this->Link_ID);
    $this->Row = 0;
    if (!$this->Query_ID) {
      $this->halt("Invalid SQL: ".$Query_String);
    }
    
    /* will return false, if query fails. This is just what we wanted. */
    return $this->Query_ID;
  }
  
  /* public: walk result set */
  function next_record() {
    if (!$this->Query_ID) {
      $this->halt("next_record called with no query pending.");
      return 0;
    }

    if ($this->Record = mssql_fetch_row($this->Query_ID)) {
      // add to Record[<key>]
      $count = mssql_num_fields($this->Query_ID);
      for ($i=0; $i<$count; $i++){
        $fieldinfo = mssql_fetch_field($this->Query_ID,$i);
        $this->Record[strtolower($fieldinfo->name)] = $this->Record[$i];
      }
      $this->Row += 1;
      $stat = 1;
    } else {
      if ($this->Auto_Free)
	      $this->free();
      $stat = 0;
    }
    return $stat;
  }

  /* public: position in result set */
  function seek($pos = 0) {
    $status = @mssql_data_seek($this->Query_ID,$pos);
    if ($status)
      $this->Row = $pos;
    else {
      $this->halt("seek($pos) failed: result has ".$this->num_rows()." rows.");
      
      /* trying to save the day */
      @mssql_data_seek($this->Query_ID, $this->num_rows());
      $this->Row = $this->num_rows();
      return 0;
    }
    
    return 1;
  }
  
  /* public: table locking */
  function lock($table, $mode="write") {
    $this->halt("How would you lock a table in mssql?");
  }
  
  function unlock() {
    $this->halt("How would you unlock a table in mssql?");
  }

  /* public: evaluate the result (size, width) */
  function affected_rows() {
    return @mssql_affected_rows($this->Query_ID);
  }
  
  function num_rows() {
    return @mssql_num_rows($this->Query_ID);
  }
  
  function num_fields() {
    return @mssql_num_fields($this->Query_ID);
  }

  function nf() {
    return $this->num_rows();
  }
  
  function np() {
    print $this->num_rows();
  }
  
  function f($Field_Name) {
    return $this->Record[strtolower($Field_Name)];
  }
  
  function p($Field_Name) {
    print $this->f($Field_Name);
  }

  /* public: sequence numbers */
  function nextid($seq_name) {
    $this->halt("This function has not been implemented. Do you know how to do it?");
    return false;
  }
  
  /* public: return table metadata */
  function metadata($table = "", $full = false) {
    $count = 0;
    $id    = 0;
    $res   = array();

    if ($table) {
      $this->connect();
      $id = @mssql_query("select * from $table", $this->Link_ID);
      if (!$id) {
        $this->halt("Metadata query failed.");
        return false;
      }
    } else {
      $id = $this->Query_ID;
      if (!$id) {
        $this->halt("No query specified.");
        return false;
      }
    }

    $count = @mssql_num_fields($id);
    
    if (!$full) {
      for ($i=0; $i<$count; $i++) {
        $info = mssql_fetch_field($id, $i);
        $res[$i]["table"] = $table;
        $res[$i]["name"]  = $info["name"];
        $res[$i]["len"]   = $info["max_length"];
        $res[$i]["flags"] = $info["numeric"];
      }
    } else {
      $res["num_fields"] = $count;
      for ($i=0; $i<$count; $i++) {
        $info = mssql_fetch_field($id, $i);
        $res[$i]["table"] = $table;
        $res[$i]["name"]  = $info["name"];
        $res[$i]["len"]   = $info["max_length"];
        $res[$i]["flags"] = $info["numeric"];
        $res["meta"][$res[$i]["name"]] = $i;
      }
    }
    if ($table)
      @mssql_free_result($id);

    return $res;
  }

  /* public: find available table names */
  function table_names() {
    $this->halt("To be modelled after the table_names() function in db_mysqlinc");
    return false;
  }

  /* private: error handling. */
  function halt($msg) {
    $this->Errno = 1;
    $this->Error = "The MSSql interface lacks proper error reporting.";
    if ($this->Halt_On_Error == "no")
      return;

    $this->haltmsg($msg);

    if ($this->Halt_On_Error != "report")
      die("Session halted.");
  }

  function haltmsg($msg) {
    printf("</td></tr></table><b>Database error:</b> %s<br>\n", $msg);
    printf("<b>MSSQL Error</b>: %s (%s)<br>\n",
      $this->Errno,
      $this->Error);
  }
}
?>
