<?php
/*
 * Query generation for PHP3
 *
 * (C) Copyright 1998-2000 Alex Aulbach
 *     if you take parts out of this class, please send
 *     a cookie-file to me - I'm collection cookies:
 *     ssilk@mayn.de :-)
 *
 *     Credits: Gerard Hickey <Gerard.Hickey@nsc.com>
 *              I took many ideas from his SQL.inc, thanks! :-)
 *              Also Credits to Carmelo Guarneri
 *              I reimplemented his db_usql.inc here
 *              hope he likes it, his functions
 *              are commented with // instead of ##
 *
 *     The idea (and also the name, sorry for this) of this class
 *     is based in November 1997,
 *     it was a collection of functions for PHP/FI and mSQL.
 *
 * $Id: query_sql.inc,v 1.1 2002/06/22 12:49:30 AlberT Exp $
 *
 */


## Query-class is ALWAYS an extension to the DB_Sql-class!
## See PHPLib-documention to use it!


class Query_SQL extends DB_Sql {

	## DONT FORGET to set the variables from DB_Sql! See there!

	## For debugging: if set to TRUE the Query is printed out,
	## before executing or returning 
	var $Q_Debug=false;

	## set this to another value, if you want to hide it
	## in your HTML-code
	## example: var $Q_Debug_print="\n<!-- QDebug: %s -->\n";
	var $Q_Debug_print="<BR><B>QDebug:</B>\n%s\n<BR>\n";

	## Set this to TRUE if you only want to test, which query
	## will be created (ideal in combination with $Q_Debug)
	## This depends only functions which will make changes
	var $No_Write=false;

	## currently unused, this var is just an idea for later use.
	var $Backslash_N_is_NULL = false;

	## Metacache: see function metadata_buffered()
	## Only needed if you are using ALTER TABLE during program runs
	var $meta_cache_off = false;

	## This is the char, which will be replaced by \char.
	## PHP3 seems to be NOT binary-safe, so not quoting
	## for \0  (some ideas?)
	## we use ereg_replace to do the replacements.
	## with PHP3.0.6 you should replace this with str_replace()!
	##   Quoting = 1 -> normal quoting using AddSlashes
	##             2 -> Replace \' to '' - needed eg. for sybase or oracle
	var $Quoting=1;
	var $Quotechar="'";
	
	var $StrLengTrunc = false;
	var $StrLengWarn = false;

	## Oracle stuff
	## Adds the behavior that an empty string is converted to NULL
	var $EmptyStringIsNull = false;
	## Adds the behavior, that NULL is converted into an single space
	## needed by some applications (SAP for example)
	var $NULLIsSpace = false;

	## Fireball-stuff
	## a regular expression that replaces chars that are 
	## listed here with '_'; this replaces all punctuation-chars
	## (;.+-/!$&/(=#*~ etc.)
	var $fireball_exclude_ereg = "[[:punct:]]";
	## and the char(s) that the found stuff will replaced with
	## another meaningful value could be '%'
	var $fireball_replace_char = "_";


	var $meta_buf=array();

	## (MORE OR LESS) PRIVATE FUNCTIONS ##


	###########################
	## _QDebug
	## ( string $str )
	function _QDebug ($str) {
		if ($this->Q_Debug) {
			printf($this->Q_Debug_print,$str);
		}
	}

	###########################
	## Set DB-Classname
	## This is only a 3rd posibility for setting the classname
	## ( string $db_class ) 
	function set_db_class ($db_class) {
		$this->Database=$db_class;
	}


	###########################
	## (array $meta)
	## : array( php_type , php_subtype )
	function sql2phptype ($meta) {
		if (!$meta['php_type'])
			$this->halt("sql2phptype(): php_type is empty!"); 
		return(Array($meta['php_type'],$meta['php_subtype']));
	}


	#######################################
	## This function returns a PHP-variable depending
	## on type. E.g. a string is returned as 'string'
	## (
	## $val - the value
	## array $meta - the meta information for this field (that's what
	##         is returned by metadata() from DB_sql-class, but just one
	##         single row, e.g. $meta[2], not hole $meta).
	## enum $special - Overwrites the type of the var if set.
	##         NULL | func | string | int | double
	## ) 
	## : array( converted SQL-value , fieldname )

	function convert ($val,$meta,$special="") {
		list($type,$subtype)=$this->sql2phptype($meta);
		if (( (string)strtoupper($val) == 'NULL' &&
		    ($type != "string" || $type[1] != "")) ||
		    (string)strtoupper($special) == 'NULL') {
			$type='NULL';
		} else {
			if ($special) {
				$type=$special;
				if ($type!="func") {
					$val=$type;
					$type="func";
				}
			}
		}
		switch ((string)$type) {
			case "string" :
				$val=(string)$val;
				if (empty($val) && $this->EmptyStringIsNull) {
					$val='NULL';
					$subtype='NULL';
				}
				if ((string)strtoupper($val)=='NULL'
				    && $this->NULLIsSpace) {
					$val=' ';
					$subtype='';
				}
				if ($this->Quoting) {
					$val=AddSlashes($val);
				}
				if ($this->Quoting==2) {
					$val=str_replace("\\'","''",$val);
				}
				if ($subtype!='date' &&
				    ( $this->StrLengTrunc || $this->StrLengWarn ) ) {
					if ( strlen($val) > $meta['len'] ) {
						if ($this->StrLengWarn) {
							echo "<BR>STRING TOO LONG: '$meta[name]'";
							if ($this->StrLengTrunc) {
								echo ", TRUNCATING!";
							}
						}
						if ($this->StrLengTrunc && (string)$subtype!='NULL') {
							$val=substr($val,0,$meta['len']);
						}
					}
				}
				if ((string)$subtype!='NULL') {
					$val=$this->Quotechar . $val . $this->Quotechar;
				}
				break;
			case "int" :
				$val=(int)$val;
				break;
			case "double" :
				$val=(double)$val;
				break;
			case "NULL" :
				$val="NULL";
				break;
			case "func" :
				$val=(string)$val;
				break;
			default :
				echo "UNKNOWN TYPE: $type<BR>";
		}
		if (!empty($meta['tableprefix'])) {
			$this->_QDebug("Val: $meta[tableprefix]$meta[name] => $val<BR>");
			return(Array($val,$meta['tableprefix'].$meta['name']));
		} else {
			$this->_QDebug("Val: $meta[name] => $val<BR>");
			return(Array($val,$meta['name']));
		}
	}

	## table_quote - converts a value of given table into a
	## corresponding correctly escaped value. Indeed it is only
	## some better type of interface for convert()
	## This is for example to create a more complex-where-clause
	## (e.g. 'date BETWEEN ... AND ...')
	## without leaving query-class. 
	## 
	function table_quote($table,$name,$value,$spec='') {
		list($table,$prefix)=$this->istableprefix($table,$spec);
		$meta=$this->metadata_buffered($table,$prefix);
		list($val,$name)=$this->convert($value,$meta[$meta['meta'][$name]],$spec);
		return($val);
	}

	##
	## Check params
	## internal function, which returns an array of splitted parameters
	## ( array or string $params )
	## : array ( array ($params) , string $check )
	function chkprms ($params) {
		if (!is_Array($params)) {
			$check=strtolower($params);
			$params=ARRAY('check'=>$check);
		} else {
			$check=strtolower(@$params['check']);
		}
		if (empty($check)) $check='soft';
		if (! ereg("strong|soft|weak",$check) ) {
			$this->halt("chkprms(): Parameter \$check is not valid: '$check'");
		}
		return(ARRAY($params,$check));
	}

	##
	## This functions checks out the situation, that $p1 or $p2 is
	## the special-array and $p2 or $p1 is a where-Parameter
	## perhaps this could be done better, but I made this in 2 minutes.
	## ( array or string $p1, array or string $p2 )
	## : array ( array ( $special) , string $where )
	function special_or_where ($p1,$p2='') {
		$special=ARRAY();
		$where='';
		if (empty($p1)) {
			if (is_Array($p2)) {
				$special=$p2;
			} elseif (strlen($p2) > 0) {  # simplest possible where clause is a=1
				$where=$p2;
			}
		} elseif (empty($p2)) {
			if (is_Array($p1)) {
				$special=$p1;
			} elseif (strlen($p1) > 0) {
				$where=$p1;
			}
		} elseif (empty($p1) && empty($p2)) {
			# do nothing
		} else {
			if (is_Array($p1) && (strlen($p2) > 0)) {
				$special=$p1;
				$where=$p2;
			} elseif (is_Array($p2) && (strlen($p1) > 0)) {
				$special=$p2;
				$where=$p1;
			} else {
				$this->halt("special_or_where(): parameters in a unkown form.");
			}
		}
		return(ARRAY($special,$where));
	}

	##
	## uniform where-clause
	## brings the where clause form
	## " WHERE blablablabla"; which is used everywhere in Query
	## ( string $where )
	## : string $where
	function uniform_where ($where) {
		$where=trim($where);
		if (!empty($where)) {
			if (!eregi("^WHERE[[:space:]]",$where)) {
				## insert "WHERE" if not set
				$where="WHERE $where";
			}
			$where=" $where";
		}
		return($where);
	}
	function uniform_add_where ($where) {
		return(eregi_Replace("^[[:space:]]*WHERE","",trim($where)));
	}

	##
	## uniforms $fields with $special
	## returns an array of values and (converted) names
	## Perhaps we can use $special for renaming those names?
	## ( array $meta , array $fields , array $special )
	## : array (array ($vals) , array($names) )
	function uniform_vars (&$meta,&$fields,&$special) {
		$vals=array(); $names=array();
		for (reset($fields); list($key,$val)=each($fields); ) {
			if ( isset($meta['meta'][$key]) ) {
				$j=$meta['meta'][$key];
				list($vals[],$names[])=
				     $this->convert($val,$meta[$j],
					(isset($special[$key])) ? $special[$key] : '');
			}
		}
		# add information for $special-fields which are not used
		# in the above loop
		for (reset($special); list($key,$val)=each($special); ) {
			if ( isset($meta['meta'][$key]) && !isset($fields[$key]) ) {
				$j=$meta['meta'][$key];
				list($vals[],$names[])=
				     $this->convert($fields[$key],$meta[$j],$val);
			}
		}
		return(ARRAY($vals,$names));
	}

	##
	## metadata_buffered
	## This function calls metadata() if it won't find the buffer
	## (string $table)
	## : array ($meta)
	function metadata_buffered($table,$prefix=false) {
		if (!$prefix) {
			if ( !isset($this->meta_buf[$table]) or
			     !is_Array($this->meta_buf[$table]) or $this->meta_cache_off) {
				return ($this->meta_buf[$table]=$this->metadata($table,true));
			} else {
				return ($this->meta_buf[$table]);
			}
		} else {
			$table=substr($table,0,strlen($table)-1);
			if ( !isset($this->meta_buf[$table]) or
			     !is_Array($this->meta_buf[$table]) or $this->meta_cache_off) {
				$this->meta_buf[$table]=$this->metadata($table,true);
			}
			for (reset($this->meta_buf[$table]) ; list($k,$v)=each($this->meta_buf[$table]) ; ) {
				if (isset($v['table']))
					$this->meta_buf[$table][$k]['tableprefix']="$v[table].";
			}
			return($this->meta_buf[$table]);
		}
	}

	##
	## istableprefix($table,$special)
	## returns (list($tablename,$prefix))
	##
	function istableprefix($table,$special) {
		$tableprefix=false;
		if (substr($table,-1,1)=='.') {
			$tableprefix=true;
			$table=substr($table,0,strlen($table));
		}
		if (!empty($special['tableprefix']))  $tableprefix=true;
		return(array($table,$tableprefix));
	}



	## BEGIN OF OPEN FUNCTIONS ##



	##
	## Function to generate an INSERT-Clause
	## (
	## string $table   tablename
	## array $fields   is an assoc. Array consisting out of
	##                 table_name => value-pairs
	## array $special  is an assoc. field which will commit special
	##                 handling to convert() (See there)
	## array or string $params  currently not used.
	## )
	## : string insert-clause
	function insert_Clause ($table,$fields,$special='',$params="soft") {
		if (empty($special)) $special=ARRAY();
		list($params,$check)=$this->chkprms($params);

		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);

		list($vals,$names)=$this->uniform_vars($meta,$fields,$special);

		$q=sprintf("INSERT INTO %s (%s) VALUES (%s)",
		   $table,join($names,", "),
		          join($vals,", "));
		$this->_QDebug($q);
		return($q);
	}

	##
	## Function to generate a plain INSERT-Clause
	## ("plain" means, that every field in the table will
	##  be set to a value, default is '' or 0)
	##
	## (
	## string $table
	## array $fields
	## array $special 
	## array or string $params  known values for array:
	##  check => "strong" | "soft"
	##  nullisnull => true
	## )
	## : string insert-clause
	function insert_plain_Clause ($table,$fields,$special='',$params="soft") {
		if (empty($special)) $special=ARRAY();
		list($params,$check)=$this->chkprms($params);

		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);

		if (Count($fields)!=$meta['num_fields'] && $check=='strong') {
			$this->halt("WARNING: insert_plain_clause(): There are not the same number of".
			     " fields as in table for INSERT");
		}

		for ($i=0; $i < $meta['num_fields']; $i++) {
			$j=$meta[$i]['name'];
			if (!isset($fields[$j])) {
				if (!empty($params['nullisnull'])) {
					$special[$j]='NULL';
				} else {
					$fields[$j]='';
				}
			}
		}
		return($this->insert_Clause($table,$fields,$special,$params));
	}

	##
	## update_clause()
	## The where parameter should be generated by yourself or will
	## be generated via unique_where_Clause() out of $fields, if not given
	## (
	## array $table, array $fields,
	## array or string $special_or_where,
	## array or string $where_or_special,
	## array or string $params - known values
	##  check => "strong" | "soft" | "weak"
	##  weak enables you to sent a query without $where (enables you
	##  to update the hole table)
	##  soft (default) takes the $fields-parameter to generate a WHERE-clause
	## )
	## : string update-clause
	##
	function update_Clause ($table,$fields,
	                        $special_or_where='',$where_or_special='',$params="soft") {
		list($special,$where)=$this->special_or_where($where_or_special,$special_or_where);
		list($params,$check)=$this->chkprms($params);

		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);

		if (empty($where) && $check=="soft") {
			$where=$this->unique_where_Clause($table,$fields);
			$check='strong';
		}
		if (empty($where) && $check=='strong') {
			echo "ERROR: update_Clause(): Parameter \$where is empty!<BR>";
			return(false);
		}
		list($vals,$names)=$this->uniform_vars($meta,$fields,$special);

		$unique_key=array_flip(explode(' ',$meta['unique']));
		for ($i=0 ; $i < Count ($names); $i++ ) {
			if (!isset($unique_key[$names[$i]]))
				$s[]=$names[$i].'='.$vals[$i];
		}
		if (Count($s)) {
			$q=sprintf("UPDATE %s SET %s%s",$table,join($s,", "),
			           $this->uniform_where($where));
		}
		$this->_QDebug($q);
		return($q);
	}

	##
	## This function is nearly the same, as update_Clause()
	## (
	## array $table, array $fields,
	## array or string $special_or_where,
	## array or string $where_or_special,
	## array or string $params - known values:
	##  check => "strong" | "soft" | "weak"
	##  weak enables you to sent a query without $where (enables you
	##  to update the hole table)
	##  soft (default) takes the $fields-parameter to generate a WHERE-clause
	##  nullisnull => true
	##         instead of updateing '' or 0 if the field is not set
	##         it will write NULL
	## )
	## : string update-clause
	##
	function update_plain_Clause ($table,$fields,
	         $special_or_where='',$where_or_special='',$params="soft") {
		list($special,$where)=$this->special_or_where($where_or_special,$special_or_where);
		list($params,$check)=$this->chkprms($params);

		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		if ((Count($fields)!=$meta['num_fields']) && $check=="strong") {
			echo "WARNING: update_plain_Clause(): This is not the correct number of".
			     " fields for UPDATE<BR>";
		}

		for ($i=0; $i < $meta['num_fields']; $i++) {
			$j=$meta[$i]['name'];
			if (!isset($fields[$j])) {
				if ($params['nullisnull']) {
					$special[$j]='NULL';
				} else {
					$fields[$j]='';
				}
			}
		}
		return($this->update_Clause($table,$fields,$special,$where,$params));
	}


	##
	## DELETE
	## deletes fields from the selected Table
	## (
	## array $table: The table
	## array $where_or_wfields
	##         Either a where-clause-string or an assoc array
	##         of fields. In this case the function tries to find an
	##         unique-identifier of those fields and creates only
	##         an where-clause for those unique-fields, otherwise
	##         for all the given fields (takes where_Clause() for this)
	## $params: Assoc Array
	##  check => "strong" | "soft" | "weak"
	##         "weak" let's you delete the hole table, if you want
	##         (so $where_or_wfields can be empty)
	##         "strong" checks if $where_or_wfields describes minimum
	##         one unique-identifier
	## )
	## : string delete-clause
	##
	function delete_Clause ($table,$where_or_wfields='',$params="soft") {
		list($fields,$where)=$this->special_or_where($where_or_wfields);
		list($params,$check)=$this->chkprms($params);

		if (empty($where)) {
			## then fields must hold some values
			list($table,$prefix)=$this->istableprefix($table,$params);
			$meta=$this->metadata_buffered($table,$prefix);
			$uniq=split(" ",$meta['unique']);
			$where=ARRAY();
			for ( reset($uniq) ; list(,$name)=each($uniq) ; ) {
				if (isset($fields[$name]))
					$where[$name]=$fields[$name];
			}
			if ($check=="strong" && !count($where)) {
				echo "ERROR: delete_Clause(): A unique identifier dosn't exist.<br>";
				echo "Turn off check='strong' or call it with wfields-params.";
				return(false);
			} elseif (count($where)) {
				$fields=$where;
			}
			$where=$this->where_clause($table,$fields);
		}

		if (!$where && $check!="weak") {
			echo "ERROR: delete_Clause(): Parameter \$where is empty!<BR>";
			return(false);
		}

		$q=sprintf("DELETE FROM %s%s",$table,
		            $this->uniform_where($where));
		$this->_QDebug($q);
		return($q);
	}


	
	##
	## WHERE-CLAUSE
	##
	## This function generates a where-clause beginning with " WHERE "
	## Different form where_plain_Clause() this function is fully automated
	## It can handle NULL-Values (IS NULL) in a special manner:
	## if a value of $fields is 'NULL', we are looking, if the
	## operator is '='. In this case the operator is changed into "IS"
	## in any other case it is changed into "IS NOT". (won't match
	## 'like' etc. but this function is not thought for such operations!)
	## (
	## string $table
	## array $fields    Assoc name=>value-fields
	## string or array $op Assoc name=>operator. If empty, '=' is taken. it is printed
	##            *between* the name/value pairs.
	##            if $op is 'func' the name is taken as function name,
	##            inside the brakets is the value.
	## $special   Affects the calculation of value.
	##            See INSERT_CLAUSE() for more about this.
	## $andor     This string is printed bewtween the name/value-pairs,
	##            default is 'AND'. If $where is set, it prints
	##            it directly at the end before concatenating
	## $where     an existing WHERE-clause. Default is empty.
	## $check     if 'strong', it will stop, if an empty where-clause
	##            will be returned, to avoid "full" selects. Default is soft
	##
	## Returns a where-clause beginning with " WHERE "
	##
	function where_Clause ($table,$fields,$op='',$special='',
	                        $andor='AND',$where='',$params="soft") {
		list($params,$check)=$this->chkprms($params);
		if (!is_Array($special)) $special=ARRAY();
		if (!$andor) $andor='AND';
		if (!is_Array($fields)) $fields=ARRAY();
		if (!is_string($where)) echo "WARNING: WHERE is not string\n";

		$q='';
		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		list($vals,$names)=$this->uniform_vars($meta,$fields,$special);

		for ($i=0; $i < Count($names) ; $i++) {
			if (is_array($op)) {
				if (empty($op[$names[$i]])) $o='='; else $o=$op[$names[$i]];
			} else {
				if (empty($op)) $o='='; else $o=$op;
			}
			if ((string)$vals[$i]=='NULL') {
				if ($o=='=' || strtoupper($o)=='IS') $o = 'IS';
				else         $o = 'IS NOT';
			}
			$q.=" $andor $names[$i] $o $vals[$i]";
		}
		if ($where) {
			$where=$this->uniform_add_where($where);
			if ($where) $q.=" $andor $where";
		}
		if (!$q && $check=='strong') {
			echo "WARNING: where_Clause(): WHERE-clause is empty!<BR>";
		}
		$q=$this->uniform_where(ereg_Replace("^ $andor ","",$q));
		$this->_QDebug("where_Clause(): $q");
		return($q);
	}

	##
	## UNIQE-WHERE-CLAUSE
	## Result is identical to where_Clause(), but it checks and uses
	## only unique-identificators and if your table has
	## none it returns an error.
	##  $special:
	##  removenull => true
	##  Removes parts that will lead to "bla IS NULL" or "bla IS NOT NULL"
	##  cause primary or unique keys cannot be defined as NULL
	##
	function unique_where_Clause ($table,$fields,$op='',$special='',
	                        $andor='AND',$where='',$params="soft") {
		list($params,$check)=$this->chkprms($params);
		if (!is_Array($special)) $special=ARRAY();
		if (!$andor) $andor='AND';
		if (!is_Array($fields)) $fields=ARRAY();

		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		if ( empty($meta['unique']) ) {
			echo "ERROR: unique_where_Clause(): Table '$table' has no unique".
			     " identifier or primary key!<br>";
			return(false);
		}
		$uniq=split(" ",$meta['unique']);
		$wfields=ARRAY();
		# loops through found uniques and remember them in $wfields
		for ( reset($uniq) ; list(,$name)=each($uniq) ; ) {
			if (!isset($fields[$name])) continue;
			if (isset($meta['meta'][$name]) ) {
				$tmp= $this->convert(@$fields[$name],
				                     $meta[$meta['meta'][$name]],
				                     @$special[$name]);
				if ( (string)$tmp[0] !='NULL' and
				     empty($special['removenull']) ) {
					$wfields[$name]=$fields[$name];
				}
			} else {
				echo "ERROR: unique_where_Clause(): This unique index dosn't exist: '$name'<br>";
				return(false);
			}
		}
		$whereclause=$this->where_Clause($table,$wfields,$op,$special,
		             $andor,$where,$special);
		if (!$whereclause) {
			echo "ERROR: unique_where_Clause(): The result of generating a where-clause to this uniqe identifers is empty:<br>";
			for ( reset($uniq); list(,$name)=each($uniq) ; ) {
				if (!isset($fields[$name]) )
					echo "\$fields[$name] is empty";
				else	echo "\$fields[$name]=>'$fields[$name]'<br>";
			}
			echo "<br>";
			return(false);
		}
		return($whereclause);
	}

	##
	## SIMP(LE) WHERE-CLAUSE
	## Formerly known as where_plain_Clause()
	## Let you generate a WHERE-Clause with a Loop.
	##
	## Returns a where-clause beginning with " WHERE "
	##
	## This function generates a where-clause
	## $mywhere   An array of simple expressions, eg. "firstname='Alex'"
	## $andor     This string is printed bewtween the where-Array
	##            default is 'AND'. It will handle an existing
	##            $where-param correctly.
	## $where     an existing WHERE-clause. Default is empty.
	## $check     if 'strong', it will stop, if an empty where-clause
	##            will be returned, to avoid "full" selects. Default is soft
	##
	function simp_where_Clause ($mywhere,$andor='AND',$where='',$check="soft") {
		$q=array();

		for ($i=0; $i<Count($mywhere); $i++ ) {
			$q2=$this->uniform_add_where($mywhere[$i]);
			if ($q2) $q[]=$q2;
		}
		if ($where) {
			$where=$this->uniform_add_where($where);
			if ($where) $q[]=$where;
		}
		if (!$q && $check=='strong') {
			$this->halt("where_plain_Clause(): WHERE-clause is empty!");
		}
		$q2=$this->uniform_where(join(" $andor ",$q));
		$this->_QDebug("where_plain_Clause(): $q2");
		return($q2);
	}



	##
	## IU - INSERT or update UPDATE query for
	##      an unique row in a table
	##
	## This function inserts or update queries, dependig
	## if the unique or primary key was found in the table or not.
	## IMPORTANT: Cause it is currently not possible to create a Query
	##            class in the Query class ($classname is not set)
	##            we must overwrite this class!
	##            The result before calling this function is not saved!
	## $table: The table
	## $fields: Assoc array of values to insert or update
	## $special: Assoc array for special handling of $fields
	## $params:  Assoc array or a string
	##   check => 'soft' | 'strong'
	##            the parameter is taken over to 
	##   forceinsert => true    
	function IU_Clause ($table,$fields,$special='',$params='soft') {
		if (empty($special)) $special=ARRAY();
		list($params,$check)=$this->chkprms($params);

		if (!$whereclause=
		     $this->unique_where_Clause($table,$fields,'',$special,$params) ) {
			echo "ERROR: IU_Clause(): Where-Clause could't generated";
			return(0);
		}

  		$this->query("select count(*) from $table $whereclause");
		$this->next_record();
		if ($this->Record[0] > 1) {
			## this error cannot happen by normal, but perhaps something
			## goes wrong with generating the where-clause?
			echo "ERROR: IU_Clause(): This index isn't unique: '$whereclause'<BR>";
			return(false);
		} elseif (empty($params['forceinsert']) && $this->Record[0] == 1) {
			return(
		 	$this->update_Clause($table,$fields,$special,$whereclause,$check) );
		} else {
			return(
			$this->insert_Clause($table,$fields,$special,$check) );
		}
	}




	##
	## This function checks wether in table $table a
	## field $name is set with value $val
	##
	## it returns the number of found matches or zero
	##
	function exists ($table,$name_or_fields,$val='',$special='') {
		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		list($fields,$name)=$this->special_or_where($name_or_fields);

		if (empty($name)) {
			$where=$this->where_Clause($table,$fields,'',$special,'','','strong');
		} elseif ($name && !empty($val) ) {
			$where=$this->where_Clause($table,ARRAY($name=>$val),'',$special,'','','strong');
		} else {
			$this->halt("exists(): Val was not set");
		}

		$q=sprintf("SELECT COUNT(*) as c FROM %s%s",
		   $table,$where);
		$this->_QDebug($q);
		$this->query($q);
		$this->next_record();
		return($this->f("c"));
	}
	## This function creates a query like exists, but returns
	## an assoc array of the first found row, or false if nothing found
	## field $name is set with value $val
	##
	function getrow ($table,$name_or_fields,$val='',$special='') {
		if (!$res=$this->getexrow(&$table,&$name_or_fields,&$val)) {
			echo "<BR><B>WARNING:</B> getrow(): '$where' not found<BR>";
			return(false);
		}
		return($res);
	}
	## This function creates a query like exists, but returns
	## an assoc array of the first found row, or false if nothing found
	## field $name is set with value $val
	##
	function getexrow ($table,$name_or_fields,$val='',$special='') {
		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		list($fields,$name)=$this->special_or_where($name_or_fields);

		if (empty($name)) {
			$where=$this->where_Clause($table,$fields,'',$special,'','','strong');
		} elseif ($name && !empty($val) ) {
			$where=$this->where_Clause($table,ARRAY($name=>$val),'',$special,'','','strong');
		} else {
			$this->halt("getexrow(): Val was not set:
'$name_or_fields' => '$val'");
		}
		$q=sprintf("SELECT * FROM %s%s",
		   $table,$where);
		$this->_QDebug($q);
		$this->query($q);
		if ($this->next_record()) {
			return($this->Record);
		} else {
			return(false);
		}
	}

	## This function is like getexrow(), and returns 
	## an assoc array of the first found row, or false if nothing found
	## If $where_or_wfields is an array it will run through
	## where_Clause() and $where will be added, otherwise the
	## where-parameter will be ignored!
	##
	function getrow_where ($table,$where_or_wfields,$special='',$where='') {
		list($fields)=$this->special_or_where($where_or_wfields);

		if (is_array($fields))
			$where=$this->where_Clause($table,$fields,'',
			       $special,'',$where);

		list($table)=$this->istableprefix($table,$special);
		$q=sprintf("SELECT * FROM %s %s",
		   $table,$this->uniform_where($where));
		$this->_QDebug($q);
		$this->query($q);
		if ($this->next_record()) {
			$this->_QDebug("getrow_where(): found");
			return($this->Record);
		} else {
			$this->_QDebug("getrow_where(): not found");
			return(false);
		}
	}


	##
	## capture-vars
	##
	## This function returns an assoc. Array consisting out of
	## name=>value-pairs needed by all the other functions. It reads
	## the name of the vars from the fields in $table and the values
	## from the $GLOBALS-var-field.
	##
	## $table     The name of the table
	##
	function capture_vars ($table,$special='') {
		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);
		$r=Array();
		for ($i=0; $i < $meta['num_fields']; $i++) {
			$j=$meta[$i]['name'];
			if (isset($GLOBALS[$j])) {
			    $r[$j] = $GLOBALS[$j];
				$this->_QDebug("Found $j: $r[$j]");
			}
		}
		return($r);
	}

	##
	## capture_table_array
	##
	## Returns an assoc array, wich will hold only those name=>value
	## pairs wich are in the given table
	##
	function capture_table_array ($table,$arr) {
		$meta=$this->metadata_buffered($table);
		$r=Array();
		for ($i=0; $i < $meta['num_fields']; $i++) {
			$j=$meta[$i]['name'];
			if (isset($arr[$j])) {
			    $r[$j] = $arr[$j];
				$this->_QDebug("Found $j: $r[$j]");
			}
		}
		return($r);
	}

	//--------------------------------------------------------------
	//  this function can be used to export all the columns of
	//  a record into *global* variables.
	//  So it is the oposite function to capture_vars()
	//  It should be used after a call to next_record().
	//
	//  $param is an optional assoc-array-parameter:
	//  to_upper=>true - uppercases all vars, otherwise lowercase
	//--------------------------------------------------------------
	function dispose_vars($params='') {
		if (!is_array($params)) $params=ARRAY();
		if ( !is_Array($this->Record)) {
			$this->_QDebug("No record available!");	
			return(0);
		}
		for ( reset($this->Record); list($key,$val)=each($this->Record); ) :
			if (ereg("[A-Za-z_][A-Za-z0-9_]*", $key)) {
				if ($params[to_upper]) {
					$field_name = strtoupper($key);
				} else {
					# Default behavior, cause phplib
					# uses always lowercase
					$field_name = strtolower($key);
				}
				$this->_QDebug("'$field_name' => '$val'");
				$GLOBALS[$field_name]=$val;
			} else {
				$this->_QDebug("Not matching: '$key'");
			}
		endfor;
	}

	// Alias to dispose_vars for compatibility to db_usql
	function import_record_vars($params='') {
		$params['to_upper']=true;
		$this->dispose_vars($params);
	}

	// Alias to dispose_vars, but next_record() is called before
	function dispose_next_record($params='') {
		$this->next_record();
		$this->dispose_vars($params);
	}
	//--------------------------------------------------------------
	//  this function can be used to export all the columns of
	//  an assoc array *global* variables.
	//  The names of the indizes are the name of the global var
	//  So it is nearly the same function as dispose_vars()
	//  and another oposite function to capture_vars()
	//
	//  $param is an optional assoc-array-parameter:
	//  to_upper=>true - uppercases all vars, otherwise lowercase
	//--------------------------------------------------------------
	function dispose_asoc_array($array,$params='') {
		if (!is_array($params)) $params=ARRAY();
		for ( reset($array); list($key,$val)=each($array); ) :
			if (ereg("[A-Za-z_][A-Za-z0-9_]*", $key)) {
				if ($params['to_upper']) {
					$field_name = strtoupper($key);
				} else {
					# Default behavior, cause phplib
					# uses always lowercase
					$field_name = strtolower($key);
				}
				$this->_QDebug("'$field_name' => '$val'");
				$GLOBALS[$field_name]=$val;
			} else {
				$this->_QDebug("Not matching or empty: '$key'");
			}
		endfor;
	}
		


	##
	## all_changed_vars
	##
	## This function returns an assoc. Array consisting out of
	## name=>value-pairs which have a different value from the value
	## currently existing in your table.
	##
	## BUGS: Cannot handle NULL values correctly
	##
	## $table     The name of the table
	## $fields    Your assoc value field, which you want to check for
	## $where     The where-clause, which matches your row.
	##            This functions writes warnings, if your where-clause
	##            returns more than one row or nothing
	##
	function all_changed_vars ($table,$fields,$where='',$check='soft') {
		list($table,$prefix)=$this->istableprefix($table,$special);
		$meta=$this->metadata_buffered($table,$prefix);

		$q1="SELECT * FROM $table $where";
		$this->query($q1);
		$r=Array();
		## TODO: uniform_vars missing here
		if ($this->next_record()) {
			for ($i=0; $i < $meta['num_fields']; $i++) {
				$j=$meta[$i]['name'];
				if ($this->Record[$j]!=$fields[$j]) {
					$r[$j]=$fields[$j];
					$this->_QDebug("Changed $j:  ".$fields[$j]." -> ".$this->Record[$j]);
				}
			}
			if ($this->next_record()) {
				echo "ERROR: all_changed_vars(): Found more than one row!<BR>";
			}
		} elseif ($check!='soft') {
			echo "<BR><B>WARNING:</B> all_changed_vars(): No row found!<BR>";
		}
		$this->_QDebug("WHERE: $where");
		return($r);
	}


	//--------------------------------------------------------------
	//
	// !!!!!!!!!!!!!!!THIS FUNCTION DOSN'T WORK CORRECT!!!!!!!!!!!!!
	//
	//  This function can be used to "copy" all the records of
	//  a former query to another table
	//  you can hand over a translation-table, which is an assoc-array:
	//    this_query_column=>my_tablename_column
	//  renames this_query_column into my_table_column
	//--------------------------------------------------------------
	function copy_result($table,$translate='') {
		if(!is_array($translate)) $translate=Array();
		while ($this->next_record()) {
			list($table,$prefix)=$this->istableprefix($table,$special);
			$meta=$this->metadata_buffered($table,$prefix);
			for ( $i=0; $i < $meta['num_fields']; $i++) {
				$j=$meta[$i]['name'];
				if ($translate[$j]) {
					$v[$translate[$j]] = $this->Record[$j];
				} else {
					$v[$j] = $this->Record[$j];
				}
			}
			$this->query($this->insert_plain_clause($table,$v));
		}
	}

	// compatibility function to db_usql
	function dump_query($tablename,$bla='') {
		$this->copy_result($tablename);
	}


	//-------------------------------------------------------------
	function fireball_Clause ($string,$query,$maxwords=0) {

		## Part 1: Normalisation of $string
		## replace any tab or other char with space and remove trailing
		## backspace from '"'-char.
		$string=ereg_Replace("[[:space:]]"," ",
		        str_Replace('\\"','"',trim($string)));
		## It replaces '"hugo bla"' with 'hugo_bla'
		## The first version of this part worked with regular expressions
		## But this version seems to be faster and more stable
		$tmp=' '; $mod=0;
		for ($i=0 ; $i < strlen($string) ; $i++ ) {
			$j=substr($string,$i,1);
			if ( '"'==(string)$j ) {
				if (!$mod && ereg("[ +-]",$tmp) ) {
					$mod=1; $merker=$i;
				}
				elseif ($mod &&
				  ( $i+1 >= strlen($string) || substr($string,$i+1,1)==' ')) {
					$string=substr($string,0,$i) . substr($string,$i+1);
					$string=substr($string,0,$merker) .
					        substr($string,$merker+1);
					$mod=0; $i--;
				}
			}
			elseif ((string)$j==' ' && $mod) {
				$string=substr($string,0,$i) .$this->fireball_replace_char.
				        substr($string,$i+1);
			}
			$tmp=$j;
		}


		## Part 2: Tokenising
		## replace multiple spaces into one and split it up
		$string=ereg_Replace(" +"," ",$string);
		$s=split(" ",$string);

		## Part 2.a: Too much words?
		if ($maxwords) {
			if ($count($s) > $maxwords) {
				Return(ARRAY(false,1));
			}
		}

		## Part 2.b: UNIMPLEMENTED - this is just a comment for later use
		## A check about words that already containing other words
		## but thats difficult, think about:  '+cars car -"cable car"'
		## so I introduce it when I really need it :-)

		## Part 3: put the tokens into drawers and replace unwanted chars
		for (reset($s) ; list($cnt,$val)=each($s) ; ) {
			switch (substr($val,0,1)) {
				case '+':
					$t[$cnt]="+";
					$s[$cnt]=ereg_Replace($this->fireball_exclude_ereg,
					         $this->fireball_replace_char,substr($val,1));
					break;
				case '-':
					$t[$cnt]="-";
					$s[$cnt]=ereg_Replace($this->fireball_exclude_ereg,
					         $this->fireball_replace_char,substr($val,1));
					break;
				default:
					$t[$cnt]="1";
					$s[$cnt]=ereg_Replace($this->fireball_exclude_ereg,
					         $this->fireball_replace_char,$val);
			}
		}

		## Part 4: Creating a pseudoquery
		## smarter persons than me can of course put part 3-5 into
		## one part, but this code wouldn't be understandable any more :)
		for ( reset($s) ; list($cnt)=each($s) ; ) {
			switch ($t[$cnt]) {
				case '+':
					$q[]=$cnt;
					break;
				case '-':
					$q[]="-$cnt";
					break;
				case '1':
					unset($tmp);
					for ( $i=0 ; $i<Count($s) ; $i++ ) {
						switch ($t[$i]) {
							case "1":
								$t[$i]=2;
							case "+":
								$tmp[]=$i;
						}
					}
					$q[]="( ". join(" OR ",$tmp) ." )";
			}
		}
		$p=split(" ",join(" AND ",$q));  # not bad, eh? :)
		$m=$p;              #  to generate a understandable output

		## Part 5: reparse pseudoquery back to a query
		for ( $cnt=0; $cnt<Count($p) ; $cnt++ ) {
			$val=$p[$cnt];
			if (ereg("(-?)([[:digit:]]+)",$val,$regs)) {
				if ($regs[1]) {
					$o=" NOT ";
				} else {
					$o="";
				}
				$tmp=$s[$regs[2]]; ## Takes string no. $regs[2] for replacement
				$p[$cnt]=$o.
				         str_Replace("{s}",$tmp,$query);
				$m[$cnt]="$o '$tmp'";
			}
		}

		# Remove unused brakets
		$l=ereg_Replace("^\((.*)\)$","\\1",join(" ",$m) );

		$this->_QDebug("Where-Part: '".join(" ",$p)."'<br>What: $l");

		return(Array(join(" ",$p),$l) );
	}

}

?>
