<?php
/* shm.inc.php
 *
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; either version 2 of the License.       |
 * +----------------------------------------------------------------------+
 * | Authors: Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: shm.inc.php,v 1.3 2002/06/19 19:03:02 AlberT Exp $
 */

// This module gives primitives to manage shared memory

define('CACHE_SHM_SIZE',        4096000);
define('CACHE_SHM_PERM',        0700);
define('CACHE_SHM_MAX_PAGES',   1000);
define('CACHE_SHM_IDX_COLS',    9);

define('CACHE_SHM_IDX_KEY',     0x1a1b3971);	// MUST be an 8 digits number !!!
define('CACHE_SHM_DEFAULT_KEY', 0x1a1b3972);	// The segmet for the DEFAULT group
                                            	//   you HAVE to define here a segment
                                            	//   for each group you want for your
                                            	//   caching system.
                                            	//   i.e. GRAPHS, FUNCTIONS, etc...

define('CACHE_SHM_SEM_KREAD',   0x1a1b2a01);	// reading sem
define('CACHE_SHM_SEM_KWRITE',  0x1a1b2a02);	// writing sem
define('CACHE_SHM_SEM_KMUTEX',  0x1a1b2a03);	// mutex sem for critical sections
define('CACHE_SHM_SEM_RSLEEP',  sem_get(CACHE_SHM_SEM_KREAD, 1, CACHE_SHM_PERM));
define('CACHE_SHM_SEM_WSLEEP',  sem_get(CACHE_SHM_SEM_KWRITE, 1, CACHE_SHM_PERM));
define('CACHE_SHM_SEM_MUTEX',   sem_get(CACHE_SHM_SEM_KMUTEX, 1, CACHE_SHM_PERM));

define('CACHE_SHM_RWAIT',       CACHE_SHM_MAX_PAGES + 101);		// keys for locking vars
define('CACHE_SHM_WWAIT',       CACHE_SHM_MAX_PAGES + 102);
define('CACHE_SHM_RCOUNT',      CACHE_SHM_MAX_PAGES + 103);
define('CACHE_SHM_WCOUNT',      CACHE_SHM_MAX_PAGES + 104);


define('CACHE_SHM_SEM_RMODE',   1);
define('CACHE_SHM_SEM_WMODE',   2);

define('CACHE_SHM_IDX_SIZE',    (CACHE_SHM_IDX_COLS * 45 + 6) * CACHE_SHM_MAX_PAGES); // size of the index of cached pages:
                                                                                      //  CACHE_SHM_MAX_PAGES arrays of
                                                                                      //  CACHE_SHM_IDX_COLS elements.

function _sem_remove($id)
{
	if (function_exists('sem_remove')) {
		//echo 'Uso sem_remove()<br>';
		return sem_remove($id[0]);
	}
	//echo "ipcs | grep 0x".dechex($id[1])." | awk {'print \$2'}";
	$id = exec("ipcs | grep 0x".dechex($id[1])." | awk {'print \$2'}");

	return exec("ipcrm sem $id");
}


/*
if(_sem_remove(array(CACHE_SHM_SEM_READING, CACHE_SHM_SEM_KREAD)))
	cache_error('DEBUG: SEMAFORO di lettura RIMOSSO !!<br>');
*/

function _getKey($id)
{
	$nCol = 0;
	// find an ID for the row
	$nRow = FNV_hash($id) % (CACHE_SHM_MAX_PAGES/2) + CACHE_SHM_MAX_PAGES/2 - 1;
	//$nRow *= 10;									// I'll never have 10 columns :-P
	$arr = shm_get_var(CACHE_SHM_IDX_ID, $nRow);
	//var_dump($arr);
	if (count($arr) > 0) {
		//cache_error( '<br>La pagina richiesta  frutto di una collisione'.
		//             ' restituisco quella indicizzata dal suo md5<br>');
		$nCol = array_search(md5($id), $arr);
	}

	return $nRow*10 + $nCol;
}


//  Fowler / Noll / Vo (FNV) Hash
function FNV_hash($str)
{
	$prime = 16777619;
	$h = $i = 0;
	$s = strlen($str);

	while ($i<$s) {
		$h = ($h * $prime) ^ ord($str[$i++]);
	}
	return $h;
}


/***************************\
        SHM PRIMITIVES
\***************************/


// attach shm
function cache_shm_open($grp='DEFAULT')
{
	$id = shm_attach(constant("CACHE_SHM_{$grp}_KEY"), CACHE_SHM_SIZE, CACHE_SHM_PERM);
	if (!$id) {
		cache_error("ERROR: cache_shm_open() couldn't create or open shared memory segment.\n");
		return 0;
	}

	// attach shm for index
	define('CACHE_SHM_IDX_ID', shm_attach(CACHE_SHM_IDX_KEY, CACHE_SHM_IDX_SIZE, CACHE_SHM_PERM));
	if (!CACHE_SHM_IDX_ID) {
		cache_error("ERROR: cache_shm_open() couldn't create or open shared memory INDEX segment.\n");
		return 0;
	}
	return $id;
}

// detach the shared memory segment
function cache_shm_close($id)
{
	if (!shm_detach($id)) {
		cache_error("ERROR: cache_shm_close() failed ... close() before open() ?!?\n");
		return FALSE;
	}

	if (!shm_detach(CACHE_SHM_IDX_ID)) {
		cache_error("ERROR: cache_shm_close() failed detaching index ... close() before open() ?!?\n");
		return FALSE;
	}
	return TRUE;
}

// add to cache
function cache_shm_add($shm_id, $id, $cont, $threshold=0)
{
	// make the index
	$md5 = md5($id);
	// find an ID for the row
	$nRow = FNV_hash($id) % (CACHE_SHM_MAX_PAGES/2) + CACHE_SHM_MAX_PAGES/2 - 1;
	$arr = @shm_get_var(CACHE_SHM_IDX_ID, $nRow);
	if (empty($arr) || !isset($arr)) {
		$arr[0] = $md5;
		shm_put_var(CACHE_SHM_IDX_ID, $nRow, $arr);
		echo '<br>La riga  vuota ... inserisco md5<br>';
	}
	else {
		$arr[] = $md5;
		if (in_array($md5, $arr)) {
			cache_error('FATAL: collisione MD5 !!!<br>');
			return FALSE;
		}
		shm_put_var(CACHE_SHM_IDX_ID, $nRow, $arr);
		echo '<br>Ho trovato una collisione ... passo alla prima colonna vuota<br>';
	}
	$key = $nRow*10 + (count($arr)-1);			// I'll never have 10 columns :-P

	// use key index to insert var in shm
	return shm_put_var($shm_id, $key, $cont);
}

// get from cache
function cache_shm_get($shm_id, $id)
{
	return shm_get_var($shm_id, _getKey($id));
}

// delere from cache
function cache_shm_rm($shm_id, $id)
{
	return shm_remove_var($shm_id, _getKey($id));
}

// check if in cache
function cache_shm_check($shm_id, $id)
{
	return (shm_get_var($shm_id, _getKey($id))) ? TRUE : FALSE;
}

// update cached
function cache_shm_update($shm_id, $id, $cont)
{
	if ($key = _getKey($id))
		return shm_put_var($shm_id, $key, $cont);
	cache_error("cache_shm_update(): variable NOT already present, use cache_shm_add instead.\n");
	return FALSE;
}

// reset cache
function cache_shm_reset($shm_id, $group='')
{
	@shm_remove(CACHE_SHM_KEY);
	@shm_remove(CACHE_SHM_KEY+1);

	// attach shm
	$shm_id[0] = shm_attach(CACHE_SHM_KEY, CACHE_SHM_SIZE, CACHE_SHM_PERM);
	if (!$shm_id[0]) {
		cache_error("shm_open(): couldn't create or open shared memory segment.\n");
		return 0;
	}

	$shm_id[1] = shm_attach(CACHE_SHM_KEY+1, CACHE_SHM_IDX_SIZE, CACHE_SHM_PERM);
	if (!$shm_id[1]) {
		cache_error("shm_open(): couldn't create or open shared memory segment.\n");
		return 0;
	}

	return $shm_id;
}

// lock cache
function cache_shm_lock($mode)
{

//var_dump(shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT));

	switch ($mode) {
		// I am a consumer
		case CACHE_SHM_SEM_RMODE:
			sem_acquire(CACHE_SHM_SEM_MUTEX);
			$wwait = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT);
			if ($wwait > 0) {
				$rwait = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RWAIT);
				shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_RWAIT, ++$rwait);
				sem_release(CACHE_SHM_SEM_MUTEX);
				sem_acquire(CACHE_SHM_SEM_RSLEEP);
			}
			else {
				$rcount = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT);
				shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT, ++$rcount);
				sem_release(CACHE_SHM_SEM_MUTEX);
			}
			break;
		// I am a producer
		case CACHE_SHM_SEM_WMODE:
			sem_acquire(CACHE_SHM_SEM_MUTEX);
			$rcount = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT);
			if ($rcount > 0) {
				$wwait = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT);
				shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT, ++$wwait);
				sem_release(CACHE_SHM_SEM_MUTEX);
				sem_acquire(CACHE_SHM_SEM_WSLEEP);
			}
			break;
		default:
			cache_error('ERROR: cache_shm_lock, '.$mode.' non  una modalit valida.<br>');
			return FALSE;
			break;
	}
	return TRUE;
}

// unlock cache
function cache_shm_unlock($mode)
{
	switch ($mode) {
		// I am a consumer
		case CACHE_SHM_SEM_RMODE:
			sem_acquire(CACHE_SHM_SEM_MUTEX);
			$rcount = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT);
			shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT, --$rcount);
			$wwait = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT);
			if ($rcount == 0 &&  $wwait > 0) {
				shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_WWAIT, --$wwait);
				sem_release(CACHE_SHM_SEM_WSLEEP);
			}
			else {
				sem_release(CACHE_SHM_SEM_MUTEX);
			}
			break;
		// I am a producer
		case CACHE_SHM_SEM_WMODE:
			$rwait = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RWAIT);
			if ($rwait > 0) {
				while ($rwait > 0) {
					shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_RWAIT, --$rwait);
					$rcount = (int) @shm_get_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT);
					shm_put_var(CACHE_SHM_IDX_ID, CACHE_SHM_RCOUNT, ++$rcount);
					sem_release(CACHE_SHM_SEM_RSLEEP);
				}
			}
			else {
				sem_release(CACHE_SHM_SEM_MUTEX);
			}
			break;
		default:
			cache_error('ERROR: cache_shm_lock, '.$mode.' non  una modalit valida.<br>');
			return FALSE;
			break;
	}
	return TRUE;
}

?>