<?php
/*
 * +----------------------------------------------------------------------+
 * | myNews 0.5 - Author : Vincenzo Visciano <liberodicrederci@yahoo.it>  |
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; version 2 of the License.              |
 * +----------------------------------------------------------------------+
 *
 */

// config file.
require_once AES_SITE_PATH.'/packages/news/config.inc.php';

function _news_mb_ErrorHandler($details)
{
	AES_print(sprintf('%s: PIPPO %s', $details['scope'], $details['message']), E_USER_ERROR);
}

class myNews
{
	var $db         = NULL;      // the db object
	var $headQuery  = '';        // head query for db
	var $newsReaded = array();   // array with SELECT result
	var $maxNews    = 1;         // news's max number

	/**
	 *  class costructor
	 *
	 * @access public
	 * @return bool
	 */
	function myNews()
	{
		//  connecting to the DBMS
		if (!$this->_setup_db()) {
			return FALSE;
		}
		// set the news's max number
		$this->SetMaxNews();

		// head query to select from the DB all the informations about a news
		$this->headQuery = 'SELECT news.idNews, news.idAuthor, news.title, news.subtitle, '.
		                   'news.news, news.nRead, news.newsDate, news.approved, '.
		                   NEWS_TABLE_AUTHORS.'.'.NEWS_FIELD_REAL_NAME.' as realName, '.
		                   'news_Topics.idTopic, news_Topics.topic, news_Topics.topicImage '.
		                   'FROM news, news_Topics, '.NEWS_TABLE_AUTHORS.' '.
		                   'WHERE (news.idTopic = news_Topics.idTopic) '.
		                   'AND (news.idAuthor = '.NEWS_TABLE_AUTHORS.'.'.NEWS_FIELD_ID_AUTHOR.') ';

		return TRUE;
	}

	/**
	 * Add the images to news in the DB
	 *
	 * @param $description, $image array
	 * @access public
	 * @return mixed TRUE on succes or error string
	 */
	function AddImagesToNews($descriptions, $images)
	{

		if (empty($this->idNews)) {
			return FALSE;
		}

		for ($i=0; $i<count($descriptions); ++$i) {

			/* check for security */
			$description = strip_tags($descriptions[$i], '<a><b><i>');
			$description = trim(stripslashes($description));
			$image = $images[$i];

			if ($image != '') {
				$this->db->GetSequenceNextValue('news_idImageSequence', &$idImage);

				$query = "INSERT INTO news_Images (idNews, idImage, imageDescription, imageUrl) ".
				         "VALUES ('$this->idNews', '$idImage', '$description', '$image')";
				$success = $this->db->Query($query);

				if (!$success) {
					AES_print(_("Error: ").$query, E_USER_ERROR);
					return FALSE;
				}
			}
		}

		return TRUE;
	}

	/**
	 * Add the links to the news in the DB
	 *
	 * @param $descriptions, $links array
	 * @access public
	 * @return mixed TRUE on succes or error string
	 */
	function AddLinksToNews($descriptions, $links)
	{

		if (empty($this->idNews)) {
			return FALSE;
		}

		for ($i=0; $i<count($descriptions);$i++) {
			/* check for security */
			$description = strip_tags($descriptions[$i], '<a><b><i>');
			$description = trim(stripslashes($description));
			$link = $links[$i];

			if ($link != '') {
				$this->db->GetSequenceNextValue('news_idImageSequence', &$idLink);
				$query = "INSERT INTO news_Links (idNews, idLink, linkDescription, linkUrl) ".
				         "VALUES ('$this->idNews', '$idLink', '$description', '$link')";
				$success = $this->db->Query($query);
				if (!$success) {
					AES_print(_("Error: ").$query, E_USER_ERROR);
					return FALSE;
				}
			}
		}

		return TRUE;
	}

	/**
	 * Add a news in the DB
	 *
	 * @access public
	 * @return mixed TRUE on succes or error string
	 */
	function AddNews($idNews='', $idAuthor, $idTopic, $title, $subtitle, $text, $approved)
	{
		/* check for security */
		$title    = strip_tags($title);
		$title    = trim(stripslashes($title));
		$subtitle = strip_tags($subtitle, '<a><b><i>');
		$subtitle = trim(stripslashes($subtitle));
		$text     = strip_tags($text, '<a><b><i>');
		$text     = trim(stripslashes($text));

		if ($idNews == '') {
			$this->db->GetSequenceNextValue('news_idNewsSequence', &$this->idNews);
		}

		$query = "INSERT INTO news (idNews, idAuthor, idTopic, title, subtitle, news, newsDate, approved) ".
		         "VALUES ('$this->idNews', '$idAuthor', '$idTopic', '$title', '$subtitle', '$text', '".date("Y-m-d H:i:s")."', $approved)";
		$success = $this->db->Query($query);

		/*  I've tried Replace, but it doesn't work!! :(
		 *
		$fields = array( 'idNews'     => array( 'Value' => $idNews,
		                                        'Type'  => 'integer',
		                                        'key'   => 1),
		                 'idAuthor'   => array( 'Value' => $idAuthor,
		                                        'Type'  => 'text'),
		                 'idTopic'    => array( 'Value' => $idTopic,
		                                        'Type'  => 'integer'),
		                 'title'      => array( 'Value' => $title,
		                                        'Type'  => 'text'),
		                 'subtitle'   => array( 'Value' => $subtitle,
		                                        'Type'  => 'text'),
		                 'news'       => array( 'Value' => $news,
		                                        'Type'  => 'text'),
		                 'newsDate'   => array( 'Value' => "date("Y-m-d H:i:s")",
		                                        'Type'  => 'timestamp'),
		                 'approved' => array( 'Value' => $approved,
		                                        'Type'  => 'boolean')
		               );

		$success = $this->db->Replace('news', $fields); */

		if (!$success) {
			return FALSE;
		}
		return TRUE;
	}

	/**
	 * Add a new topic in the DB
	 *
	 * @access public
	 * @return mixed TRUE on succes or error string
	 */
	function AddTopic($topic, $topicDescription = '', $topicImage = '')
	{
		/* check for security */
		$topic = strip_tags(trim(stripslashes($topic)));
		$topicDescription = strip_tags(trim(stripslashes($topicDescription)));

		$this->db->GetSequenceNextValue('news_idTopicSequence', &$idTopic);

		$query = "INSERT INTO news_Topics (idTopic, topic, topicDescription, topicImage) ".
		         "VALUES ('$idTopic', '$topic', '$topicDescription', '$topicImage')";
		$success = $this->db->Query($query);


		if (!$success) {
			return FALSE;
		}
		return TRUE;
	}


	/**
	 * Read a single news
	 *
	 * @access public
	 * @param int $newsId The news ID
	 * @return associative array on succes or FALSE
	 */
	function GetById($newsId)
	{
		$query  = $this->headQuery.'AND news.idNews = "'.$newsId.'"';
		$row = array();
		$success = $this->db->QueryRow($query, &$row);
		if (!$success) {
			AES_print(_("Error getting news by id"), E_USER_WARNING);
			return FALSE;
		}
		$this->newsReaded = array (0 => $row);
		if (!$this->_newsCounter()) {
			echo "errore matornale!"; // add 1 to news's read number
		}
		return $this->newsReaded;
	}


	/**
	 * get Topics
	 *
	 * @access public
	 * @return associative Array or FALSE
	 */
	function GetTopics()
	{
		$query  = 'SELECT idTopic, topic, topicDescription, topicImage '.
		          'FROM news_Topics '.
		          'ORDER BY topic ASC';
		$Topics = array();
		$success = $this->db->QueryAll($query, &$Topics);
		if (!$success) {
			AES_print(_("Error getting news topic"), E_USER_WARNING);
			return FALSE;
		}
		return $Topics;
	}

	/**
	 * Read the news by Topic
	 *
	 * @access public
	 * @param int $topicId The topic ID
	 * @return array on success or error string
	 */
	function GetByTopic($topicId, $approved=1)
	{
		$success = '';
		$this->newsReaded = array();

		$this->db->SetSelectedRowRange(0, $this->maxNews);

		$query  = $this->headQuery."AND (news.idTopic = '$topicId') AND (news.approved = $approved)";
		$success = $this->db->QueryAll($query, &$this->newsReaded);
		if (!$success) {
			AES_print(_("Error getting news by topic"), E_USER_WARNING);
			return FALSE;
		}
		return $this->newsReaded;
	}


	/**
	 * Read the recent news
	 *
	 * @access public
	 * @return mixed array on success or error string
	 */
	function GetByRecent()
	{
		$this->db->SetSelectedRowRange(0, $this->maxNews);
		$this->newsReaded = array();

		$query  = $this->headQuery.'AND (news.approved = 1) ORDER BY newsDate DESC';
		$success = $this->db->QueryAll($query, &$this->newsReaded);
		var_dump($success);
		if (!$success) {
			AES_print(sprintf(_("Error getting news by recent: '%s'"), $this->db->error()), E_USER_WARNING);
			//return FALSE;
		}
		print_r($this->newsReaded);
		return $this->newsReaded;
	}


	/**
	 * Get news's links values
	 *
	 * @access public
	 * @return mixed associative array with link description as key and link URL as value on success or error string
	 */
	function GetNewsLinks($idNews)
	{
		$links = array();

		$query  = 'SELECT linkDescription, linkUrl FROM news_Links WHERE idNews ="'.$idNews.'"';
		$success = $this->db->QueryAll($query, &$links);
		if (!empty($links)) {
			foreach ($links as $link) {
				$myLinks ["$link[0]"] = $link[1];
			}
		} else {
			$myLinks = array('' => '');
		}

		if (!$success) {
			AES_print(sprintf(_("Error getting news links: '%s'"), $this->db->error()), E_USER_WARNING);
			return FALSE;
		}
		return $myLinks;
	}



	/**
	 * Remove  News
	 *
	 * @access public
	 * @param int $idNews
	 * @return array on success or error string
	 */
	function RemoveNews($idNews)
	{
		$success = '';
		$result = array();

		$query  = 'DELETE FROM news WHERE idNews = "'.$idNews.'"';
		$successNews = $this->db->Query($query);
		$query  = 'DELETE FROM news_Links WHERE idNews = "'.$idNews.'"';
		$successLinks = $this->db->Query($query);

		//remove images from FS and DB
		$query  = 'SELECT imageUrl FROM news_Images WHERE idNews = "'.$idNews.'"';
		$success = $this->db->QueryAll($query, &$result);
		if (!empty($result)) {
			if ($this->_removeFiles($result)) {
				$query  = 'DELETE FORM news_Images WHERE idNews = "'.$idNews.'"';
				$successImages = $this->db->Query($query);
			} else {
				AES_print(sprintf(_("Error deleting files: <pre>%s</pre>"), $result));
				return FALSE;
			}
		}

		if (!$successNews || !$successLinks || !$successImages) {
			AES_print(_("Error: ").$this->db->error(), E_USER_ERROR);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * remove the last News added
	 *
	 * @access public
	 * @return boolean
	 */
	function RemoveLastNews()
	{
		if ($this->RemoveNews($this->idNews)) {
			$this->idNews = '';
			return TRUE;
		}
		AES_print(_("Error: ").$this->db->error(), E_USER_ERROR);
		return FALSE;
	}

	/**
	 * set the news's max number
	 *
	 * @access public
	 * @return boolean
	 */
	function SetMaxNews($maxNews='')
	{
		$this->maxNews = empty($maxNews) ? NEWS_MAX_NEWS : $maxNews;

		return TRUE;
	}

	/**
	 * remove the files from the FS
	 *
	 * @access public
	 * @return boolean
	 */
	function _removeFiles($files)
	{
		foreach ($files as $file) {
			if (!unlink($file))
				return FALSE;
		}

		return TRUE;
	}

	/**
	 * _newsCounter() - Add 1 to news read number
	 *
	 * @access private
	 * @return mixed TRUE on success or error string
	 */
	function _newsCounter()
	{
		if (count($this->newsReaded) == 1) {
			$new_n_read = $this->newsReaded[0][NEWS_N_READ] + 1;
			$update = 'UPDATE news SET nRead = "'.$new_n_read.
			          '" WHERE idNews = "'.$this->newsReaded[0][NEWS_ID_NEWS].'"';
			$success = $this->db->Query($update);
			if (!$success && isset($this->db->error)) {
				AES_print(_("Error: ").$this->db->error(), E_USER_ERROR);
				return FALSE;
			}
		} else {
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Initialize the object "db" to connect to DBMS with MetaBase.
	 *
	 * @access private
	 * @return mixed TRUE on success or error string
	 */
	function _setup_db()
	{
		$error = MetabaseSetupDatabaseObject( array('Type'        => NEWS_MB_TYPE,
		                                            'User'        => NEWS_MB_USER,
		                                            'Password'    => NEWS_MB_PASS,
		                                            'Host'        => NEWS_MB_HOST,
		                                            'IncludePath' => NEWS_MB_DIR),
		                                      $this->db);
		if ($error != '') {
			AES_print(_("Error: ").$error, E_USER_ERROR);
			return FALSE;
		}

		$this->db->SetDatabase(NEWS_MB_DB);
		$this->db->SetErrorHandler('_news_mb_ErrorHandler');
		return TRUE;
	}
}
?>
