<?php
/* zb_news.inc.php
 *
 * +----------------------------------------------------------------------+
 * |                                                                      |
 * | AlberT-EasySite: Web Site Management System                          |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 * | Copyleft (c) SuperAlberT.it                                          |
 * +----------------------------------------------------------------------+
 * | This program is free software. You can redistribute it and/or modify |
 * | it under the terms of the GNU General Public License as published by |
 * | the Free Software Foundation; version 2 of the License.              |
 * +----------------------------------------------------------------------+
 * | Authors: Michel Morelli     <michel@ZioBudda.net>                    |
 * |          Vincenzo Visciano  <liberodicrederci@yahoo.it>              |
 * |          Emiliano Gabrielli <AlberT@SuperAlberT.it>                  |
 * |                                                                      |
 * +----------------------------------------------------------------------+
 *
 * $Id: zb_news.inc.php,v 1.4.2.1 2002/12/18 11:30:28 AlberT Exp $
 */

// returns if we are already been included
if (defined('_ZB_NEWS_')) {
	return FALSE;
}
define('_ZB_NEWS_', TRUE);

// dbms class
require_once AES_SITE_PATH.'/packages/zb_news/common.inc.php';

/**
 * Class dedicated to news.
 *
 * @param object $dbms An optional dbms object instance
 */
class zb_news
{
	/**
	 * Stores DBMS connection object
	 */
	var $dbms = NULL;

	/**
	 * Class constructor.
	 * It uses MetaBase as abstraction layer to connect to DBMS.
	 */
	function zb_news($dbms=NULL)
	{
		if ($dbms === NULL) {
			$err = MetabaseSetupDatabaseObject( array( 'Type'        => ZB_NEWS_MB_TYPE,
			                                           'User'        => ZB_NEWS_MB_USER,
			                                           'Password'    => ZB_NEWS_MB_PASS,
			                                           'Host'        => ZB_NEWS_MB_HOST,
			                                           'IncludePath' => ZB_NEWS_MB_DIR),
			                                    $this->dbms );
		}
		elseif (is_object($dbms)) {
			$this->dbms = $dbms;
			$err = '';
		}
		else {
			$err = _('Error in zb_news: the argument of the class constructor is not a valid object.');
		}
		if ($err != '') {
			AES_print($err, E_USER_NOTICE);
			return FALSE;
		}
		return $this->dbms->SetDatabase(ZB_NEWS_MB_DB);
	}

	/**
	 * Checks by name if a Category exists
	 *
	 * @param string $name the name of the category to check
	 */
	function catExists($name)
	{
		if (trim($name)==='' || !ereg('^[A-Za-z0-9 \-]+$', $name)) {
			return ZB_NEWS_INVALID_CAT_NAME;
		}
		$query = 'SELECT category_id FROM news_categories'.
		         "WHERE LOWER(name) = '".strtolower($name)."'";
		$res = $this->dbms->Query($query);

		$end = $this->dbms->EndOfResult($res);
		if ($end==0) {
			return TRUE;
		}
		elseif ($end==1) {
			//No cat
			return FALSE;
		}
		else {
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}
	}

	/**
	 * Saves the data. If $id=='' then creates a new one.
	 *
	 * @param string $name The name
	 * @param int $state The state to set
	 * @param int $id The unique ID.
	 */
	function catSave($name, $state, $id='', $check=TRUE)
	{
		if ($state != ZB_NEWS_ENABLED && $state != ZB_NEWS_DISABLED) {
			return ZB_NEWS_INVALID_STATE_CODE;
		}
		if (trim($name)=='' || !ereg('^[A-Za-z0-9 \-]+$', $name)) {
			return ZB_NEWS_INVALID_CAT_NAME;
		}
		if (trim($id) != '' && !ereg('^[0-9]+$', $id)) {
			return ZB_NEWS_INVALID_ID;
		}
		if ($check==TRUE && $this->exists($name)) {
			return ZB_NEWS_VALID_CAT;
		}
		if ($id != '') {
			$query = "UPDATE news SET approved = $state WHERE category_id = $id";
			$res = $this->dbms->Query($query);
			if ($res==0) { //Error
				AES_print($this->dbms->Error(), E_USER_ERROR);
				return FALSE;
			}
		}
		//$query = 'REPLACE INTO news_categories(category_id, name, enabled) '.
		//         "VALUES('$id', '$name', $state)";
		$fields = array( 'category_id' => array( 'Value' => $id,
		                                         'Type'  => 'text',
		                                         'key'   => 1),
		                 'name'        => array( 'Value' => $name,
		                                         'Type'  => 'text'),
		                 'enabled'     => array( 'Value' => $state,
		                                          'Type'  => 'text')
		               );
		$res = $this->dbms->QueryReplace('news_categories', $fields);
		if (!$res) { //Error in the query
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Gets all categories from db.
	 *
	 * @return FALSE or an array of the categories.
	 */
	function getCats()
	{
		$query = 'SELECT * FROM news_categories ORDER BY name';
		$res = $this->dbms->Query($query);
		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$end = $this->dbms->EndOfResult($res);
		if ($end==0) {
			for ($row=0; ($end=$this->dbms->EndOfResult($res))==0; $row++) {
				$arr['id'][$row]      = $this->dbms->FetchResult($res, $row, 0);
				$arr['name'][$row]    = $this->dbms->FetchResult($res, $row, 1);
				$arr['enabled'][$row] = $this->dbms->FetchResult($res, $row, 2);
			}
		}
		elseif ($end==1) {
			// No more cats
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		else {
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$this->dbms->FreeResult($res);

		return $arr;
	}

	/**
	 * Gets all enabled categories from db.
	 *
	 * @return FALSE or an array of the categories.
	 */
	function getEnabledCats()
	{
		$query = 'SELECT * FROM news_categories WHERE enabled = '.ZB_NEWS_ENABLED.' ORDER BY name';
		$res = $this->dbms->Query($query);
		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$end = $this->dbms->EndOfResult($res);
		if ($end==0) {
			for ($row=0; ($end=$this->dbms->EndOfResult($res))==0; $row++) {
				$arr['id'][$row]      = $this->dbms->FetchResult($res, $row, 0);
				$arr['name'][$row]    = $this->dbms->FetchResult($res, $row, 1);
				$arr['enabled'][$row] = $this->dbms->FetchResult($res, $row, 2);
			}
		}
		elseif ($end==1) {
			// No more cats
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		else {
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$this->dbms->FreeResult($res);

		return $arr;
	}

	/**
	 * Returns the category's data, identified by ID.
	 *
	 * @param int $id The category's identifier.
	 * @return FALSE/array
	 */
	function getCatData($id)
	{
		if (!ereg('^[0-9]+$', $id)) {
			AES_print('zb_news->getCatData(): the given ID "'.$id.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}

		$query = 'SELECT * FROM news_categories WHERE category_id = '.$id;
		$res = $this->dbms->QueryRow($query, $arr);
		if (!$res) { //Error in the query
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}

		return $arr;
	}

	/**
	 * Deletes a category
	 *
	 * @param int $id The category's identifier.
	 * @return boolean.
	 */
	function deleteCategory($id)
	{
		if (!ereg('^[0-9]+$', $id)) {
			AES_print('zb_news->deleteCategory(): the given ID "'.$id.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}

		$query = 'DELETE FROM news_categories WHERE category_id = '.$id;
		if ($this->dbms->Query($query)==0) {
			AES_print('zb_news->deleteCategory(): failed deleting Category', E_USER_NOTICE);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Saves category's data in the db.
	 *
	 * @param string $who Who inserts the new.
	 * @param string $title The new's title.
	 * @param string $link The new's title link.
	 * @param int $category The category's id of the new.
	 * @param int $state The new's state.
	 * @param int $id The new's id. If '' then the new will be created.
	 */
	function saveNew($who, $title, $new, $link, $category, $state, $id='')  ///@@@
	{
		$ret = array();

		//Checks.
		if ( strlen(trim($who))==0 || strlen(trim($title))==0 ||
		     strlen(trim($new))==0 || strlen(trim($link))==0 ||
		     !ereg('^[0-9]+$', $category)) {
			return ZB_NEWS_EMPTY_FIELD;
		}
		// my editor seems to fail highlighting, so I append the '$'
		if (!ereg("^[A-Za-z0-9' .\<\>\@\-]+".'$', $who)) {
			return ZB_NEWS_INVALID_WHO_FIELD;
		}
		if (!ereg("^[A-Za-z0-9' _\-]+".'$', $title)) {
			return ZB_NEWS_INVALID_TITLE_FIELD;
		}
		if (!ereg('^[A-Za-z0-9@:_.\/\-]+$', $link)) {
			return ZB_NEWS_INVALID_URL_FIELD;
		}

		$who   = urlencode(escapeshellcmd(trim($who)));
		$title = urlencode(trim(escapeshellcmd(stripslashes($title))));
		$new   = strip_tags($new, '<a><b><i>');
		$new   = urlencode(trim(escapeshellcmd(stripslashes($new))));
		$link  = urlencode(trim(escapeshellcmd(stripslashes($link))));

		//$query = 'REPLACE INTO news(new_id, category_id, who, date, url, new, title, approved) '.
		//         "VALUES ('$id', $category, '$who', ".time().", '$link', '$new', '$title', $state)";
		$fields = array( 'new_id'     => array( 'Value' => $id,
		                                         'Type'  => 'text',
		                                         'key'   => 1),
		                 'category_id' => array( 'Value' => $category,
		                                         'Type'  => 'text'),
		                 'who'         => array( 'Value' => $who,
		                                         'Type'  => 'text'),
		                 'date'        => array( 'Value' => time(),
		                                         'Type'  => 'text'),
		                 'url'         => array( 'Value' => $link,
		                                         'Type'  => 'text'),
		                 'new'         => array( 'Value' => $new,
		                                         'Type'  => 'text'),
		                 'title'       => array( 'Value' => $title,
		                                         'Type'  => 'text'),
		                 'approved'    => array( 'Value' => $state,
		                                         'Type'  => 'text')
		               );
		$res = $this->dbms->QueryReplace('news', $fields);

		if (!$res) { //Error in the query
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Returns data from last $number news whose state is $enab
	 *
	 * @param int $number The number of news to be returned.
	 * @param int $enab The new's state.
	 * @return associative_array.
	 */
	function getLastNewsData($number=14, $enab=0)
	{
		if (!ereg('^[0-9]+$', $number)) {
			AES_print('zb_news->getLastNewsData(): the given $number "'.$number.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}
		if (!ereg('^[0-9]+$', $enab)) {
			AES_print('zb_news->getLastNewsData(): the given $enab flag "'.$enab.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}

		$query = 'SELECT news.*, news_categories.name'.
		         ' FROM news, news_categories'.
		         ' WHERE news.category_id=news_categories.category_id'.
		         ' AND visible = 1 AND approved = '.$enab.
		         ' ORDER BY new_id DESC';
		$this->dbms->SetSelectedRowRange(0, $number);
		$res = $this->dbms->Query($query);

		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$end = $this->dbms->EndOfResult($res);

		if ($end==0) {
			for ($i=0; ($end=$this->dbms->EndOfResult($res))==0; $i++) {
				$ret['id'][$i]       = $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_NEWID);
				$ret['who'][$i]      = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_WHO)));
				$ret['date'][$i]     = date('d/m/Y', $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_DATE));
				$ret['url'][$i]      = urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_URL));
				$ret['new'][$i]      = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_NEW)));
				$ret['title'][$i]    = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_TITLE)));
				$ret['category'][$i] = $this->dbms->FetchResult($res, $i, 'name');
				$ret['redir'][$i]    = $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_REDIR);
				$ret['approved'][$i] = $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_APPROVED);
			}
		}
		elseif ($end==1) {
			// No more news
			$this->dbms->FreeResult($res);
			return ZB_NEWS_NO_NEWS;
		}
		else {
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$this->dbms->FreeResult($res);

		return $ret;
	}

	/**
	 * Returns data from new identified by $id
	 *
	 * @param int $number The number of news to be returned.
	 * @param int $enab The new's state.
	 * @return associative_array.
	 */
	function getNewData($id)
	{
		if (!ereg('^[0-9]+$', $id)) {
			AES_print('zb_news->getNewData(): the given $id "'.$id.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}

		$query = 'SELECT news.*, news_categories.name '.
		         'FROM news, news_categories '.
		         'WHERE news.category_id=news_categories.category_id '.
		         'AND new_id = '.$id;
		$res = $this->dbms->Query($query);

		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$end = $this->dbms->EndOfResult($res);

		if ($end==0) {
			$ret['id'][$i]       = $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_NEWID);
			$ret['who'][$i]      = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_WHO)));
			$ret['date'][$i]     = date('d/m/Y', $this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_DATE));
			$ret['url'][$i]      = urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_URL));
			$ret['new'][$i]      = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_NEW)));
			$ret['title'][$i]    = stripslashes(urldecode($this->dbms->FetchResult($res, $i, ZB_NEWS_NEWS_TITLE)));
			$ret['category'][$i] = $this->dbms->FetchResult($res, $i, 'name');
		}
		elseif ($end==1) {
			// No more news
			$this->dbms->FreeResult($res);
			return ZB_NEWS_NO_NEWS;
		}
		else {
			AES_print($this->dbms->Error(), E_USER_ERROR);
			$this->dbms->FreeResult($res);
			return FALSE;
		}
		$this->dbms->FreeResult($res);

		return $arr;
	}

	/**
	 * Deletes the new identified by $id
	 *
	 * @param int $id The new's identifier.
	 * @return bool A return code
	 */
	function deleteNew($id)
	{
		if (!ereg('^[0-9]+$', $id)) {
			AES_print('zb_news->deleteNew(): the given $id "'.$id.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}

		$query = 'DELETE FROM news WHERE new_id = '.$id;
		$res = $this->dbms->Query($query);
		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Increments read counter for the new
	 *
	 * @param int $id The new's identifier.
	 * @return bool A return code.
	 */
	function setNewReadCounter($id)
	{
		if (!ereg('^[0-9]+$', $id)) {
			AES_print('zb_news->deleteNew(): the given $id "'.$id.'" is invalid.', E_USER_NOTICE);
			return FALSE;
		}
		$query = "UPDATE news SET redir = redir +1 WHERE new_id = $id";
		$res = $this->dbms->Query($query);

		if ($res==0) { //Error
			AES_print($this->dbms->Error(), E_USER_ERROR);
			return FALSE;
		}

		return TRUE;
	}
}
?>
